@@ -10,16 +10,24 @@ struct _Demo
 };
 
 GtkWidget *do_button_box (void);
+GtkWidget *do_colorsel (void);
+GtkWidget *do_dialog (void);
+GtkWidget *do_drawingarea (void);
+GtkWidget *do_images (void);
 GtkWidget *do_item_factory (void);
 GtkWidget *do_menus (void);
 GtkWidget *do_panes (void);
-GtkWidget *do_dialog (void);
+GtkWidget *do_pixbufs (void);
 GtkWidget *do_textview (void);
 
 Demo testgtk_demos[] = {  { "Button Boxes", "button_box.c", do_button_box },
+  { "Color Selector", "colorsel.c", do_colorsel },
+  { "Dialog and Message Boxes", "dialog.c", do_dialog },
+  { "Drawing Area", "drawingarea.c", do_drawingarea },
+  { "Images", "images.c", do_images },
   { "Item Factory", "item_factory.c", do_item_factory },
   { "Menus", "menus.c", do_menus },
   { "Paned Widgets", "panes.c", do_panes },
-  { "Dialog and Message Boxes", "dialog.c", do_dialog },
+  { "Pixbufs", "pixbufs.c", do_pixbufs },
   { "Text Widget", "textview.c", do_textview }
 };
