@@ -29,12 +29,13 @@
 #include "gdkwindow.h"
 
 static GdkDrawable* gdk_drawable_real_get_composite_drawable (GdkDrawable *drawable,
-                                                              gint         x,
-                                                              gint         y,
-                                                              gint         width,
-                                                              gint         height,
-                                                              gint        *composite_x_offset,
-                                                              gint        *composite_y_offset);
+							      gint         x,
+							      gint         y,
+							      gint         width,
+							      gint         height,
+							      gint        *composite_x_offset,
+							      gint        *composite_y_offset);
+static GdkRegion *  gdk_drawable_real_get_visible_region     (GdkDrawable *drawable);
 
 static void gdk_drawable_class_init (GdkDrawableClass *klass);
 
@@ -70,11 +71,25 @@ static void
 gdk_drawable_class_init (GdkDrawableClass *klass)
 {
   klass->get_composite_drawable = gdk_drawable_real_get_composite_drawable;
+  /* Default implementation for clip and visible region is the same */
+  klass->get_clip_region = gdk_drawable_real_get_visible_region;
+  klass->get_visible_region = gdk_drawable_real_get_visible_region;
 }
 
 /* Manipulation of drawables
  */
 
+/**
+ * gdk_drawable_set_data:
+ * @drawable: a #GdkDrawable
+ * @key: name to store the data under
+ * @data: arbitrary data
+ * @destroy_func: function to free @data, or %NULL
+ *
+ * This function is equivalent to g_object_set_data(),
+ * the #GObject variant should be used instead.
+ * 
+ **/
 void          
 gdk_drawable_set_data (GdkDrawable   *drawable,
 		       const gchar   *key,
@@ -89,6 +104,16 @@ gdk_drawable_set_data (GdkDrawable   *drawable,
                            destroy_func);
 }
 
+/**
+ * gdk_drawable_get_data:
+ * @drawable: a #GdkDrawable
+ * @key: name the data was stored under
+ * 
+ * Equivalent to g_object_get_data(); the #GObject variant should be
+ * used instead.
+ * 
+ * Return value: the data stored at @key
+ **/
 gpointer
 gdk_drawable_get_data (GdkDrawable   *drawable,
 		       const gchar   *key)
@@ -99,6 +124,16 @@ gdk_drawable_get_data (GdkDrawable   *drawable,
                              g_quark_try_string (key));
 }
 
+/**
+ * gdk_drawable_get_size:
+ * @drawable: a #GdkDrawable
+ * @width: location to store drawable's width, or %NULL
+ * @height: location to store drawable's height, or %NULL
+ *
+ * Fills *@width and *@height with the size of @drawable.
+ * @width or @height can be %NULL if you only want the other one.
+ * 
+ **/
 void
 gdk_drawable_get_size (GdkDrawable *drawable,
 		       gint        *width,
@@ -109,6 +144,14 @@ gdk_drawable_get_size (GdkDrawable *drawable,
   GDK_DRAWABLE_GET_CLASS (drawable)->get_size (drawable, width, height);  
 }
 
+/**
+ * gdk_drawable_get_visual:
+ * @drawable: a #GdkDrawable
+ * 
+ * Gets the #GdkVisual describing the pixel format of @drawable.
+ * 
+ * Return value: a #GdkVisual
+ **/
 GdkVisual*
 gdk_drawable_get_visual (GdkDrawable *drawable)
 {
@@ -117,6 +160,16 @@ gdk_drawable_get_visual (GdkDrawable *drawable)
   return GDK_DRAWABLE_GET_CLASS (drawable)->get_visual (drawable);
 }
 
+/**
+ * gdk_drawable_get_depth:
+ * @drawable: a #GdkDrawable
+ * 
+ * Obtains the bit depth of the drawable, that is, the number of bits
+ * that make up a pixel in the drawable's visual. Examples are 8 bits
+ * per pixel, 24 bits per pixel, etc.
+ * 
+ * Return value: number of bits per pixel
+ **/
 gint
 gdk_drawable_get_depth (GdkDrawable *drawable)
 {
@@ -125,6 +178,21 @@ gdk_drawable_get_depth (GdkDrawable *drawable)
   return GDK_DRAWABLE_GET_CLASS (drawable)->get_depth (drawable);
 }
 
+/**
+ * gdk_drawable_set_colormap:
+ * @drawable: a #GdkDrawable
+ * @colormap: a #GdkColormap
+ *
+ * Sets the colormap associated with @drawable. Normally this will
+ * happen automatically when the drawable is created; you only need to
+ * use this function if the drawable-creating function did not have a
+ * way to determine the colormap, and you then use drawable operations
+ * that require a colormap. The colormap for all drawables and
+ * graphics contexts you intend to use together should match. i.e.
+ * when using a #GdkGC to draw to a drawable, or copying one drawable
+ * to another, the colormaps should match.
+ * 
+ **/
 void
 gdk_drawable_set_colormap (GdkDrawable *drawable,
                            GdkColormap *cmap)
@@ -134,6 +202,15 @@ gdk_drawable_set_colormap (GdkDrawable *drawable,
   GDK_DRAWABLE_GET_CLASS (drawable)->set_colormap (drawable, cmap);
 }
 
+/**
+ * gdk_drawable_get_colormap:
+ * @drawable: a #GdkDrawable
+ * 
+ * Gets the colormap for @drawable, if one is set; returns
+ * %NULL otherwise.
+ * 
+ * Return value: the colormap, or %NULL
+ **/
 GdkColormap*
 gdk_drawable_get_colormap (GdkDrawable *drawable)
 {
@@ -142,12 +219,28 @@ gdk_drawable_get_colormap (GdkDrawable *drawable)
   return GDK_DRAWABLE_GET_CLASS (drawable)->get_colormap (drawable);
 }
 
+/**
+ * gdk_drawable_ref:
+ * @drawable: a #GdkDrawable
+ * 
+ * Deprecated equivalent of calling g_object_ref() on @drawable.
+ * (Drawables were not objects in previous versions of GDK.)
+ * 
+ * Return value: the same @drawable passed in
+ **/
 GdkDrawable*
 gdk_drawable_ref (GdkDrawable *drawable)
 {
   return (GdkDrawable *) g_object_ref (G_OBJECT (drawable));
 }
 
+/**
+ * gdk_drawable_unref:
+ * @drawable: a #GdkDrawable
+ * 
+ * Deprecated equivalent of calling g_object_unref() on @drawable.
+ * 
+ **/
 void
 gdk_drawable_unref (GdkDrawable *drawable)
 {
@@ -471,6 +564,42 @@ gdk_draw_glyphs (GdkDrawable      *drawable,
 }
 
 
+/**
+ * gdk_drawable_get_image:
+ * @drawable: a #GdkDrawable
+ * @x: x coordinate on @drawable
+ * @y: y coordinate on @drawable
+ * @width: width of region to get
+ * @height: height or region to get
+ * 
+ * A #GdkImage stores client-side image data (pixels). In contrast,
+ * #GdkPixmap and #GdkWindow are server-side
+ * objects. gdk_drawable_get_image() obtains the pixels from a
+ * server-side drawable as a client-side #GdkImage.  The format of a
+ * #GdkImage depends on the #GdkVisual of the current display, which
+ * makes manipulating #GdkImage extremely difficult; therefore, in
+ * most cases you should use gdk_pixbuf_get_from_drawable() instead of
+ * this lower-level function. A #GdkPixbuf contains image data in a
+ * canonicalized RGB format, rather than a display-dependent format.
+ * Of course, there's a convenience vs. speed tradeoff here, so you'll
+ * want to think about what makes sense for your application.
+ *
+ * @x, @y, @width, and @height define the region of @drawable to
+ * obtain as an image.
+ *
+ * You would usually copy image data to the client side if you intend
+ * to examine the values of individual pixels, for example to darken
+ * an image or add a red tint. It would be prohibitively slow to
+ * make a round-trip request to the windowing system for each pixel,
+ * so instead you get all of them at once, modify them, then copy
+ * them all back at once.
+ *
+ * If the X server or other windowing system backend is on the local
+ * machine, this function may use shared memory to avoid copying
+ * the image data.
+ * 
+ * Return value: a #GdkImage containing the contents of @drawable
+ **/
 GdkImage*
 gdk_drawable_get_image (GdkDrawable *drawable,
                         gint         x,
@@ -486,9 +615,16 @@ gdk_drawable_get_image (GdkDrawable *drawable,
   g_return_val_if_fail (GDK_IS_DRAWABLE (drawable), NULL);
   g_return_val_if_fail (x >= 0, NULL);
   g_return_val_if_fail (y >= 0, NULL);
-  g_return_val_if_fail (width >= 0, NULL);
-  g_return_val_if_fail (height >= 0, NULL);
 
+  /* FIXME? Note race condition since we get the size then
+   * get the image, and the size may have changed.
+   */
+  
+  if (width < 0 || height < 0)
+    gdk_drawable_get_size (drawable,
+                           width < 0 ? &width : NULL,
+                           height < 0 ? &height : NULL);
+  
   composite =
     GDK_DRAWABLE_GET_CLASS (drawable)->get_composite_drawable (drawable,
                                                                x, y,
@@ -522,3 +658,58 @@ gdk_drawable_real_get_composite_drawable (GdkDrawable *drawable,
   
   return GDK_DRAWABLE (g_object_ref (G_OBJECT (drawable)));
 }
+
+/**
+ * gdk_drawable_get_clip_region:
+ * @drawable: a #GdkDrawable
+ * 
+ * Computes the region of a drawable that potentially can be written
+ * to by drawing primitives. This region will not take into account
+ * the clip region for the GC, and may also not take into account
+ * other factors such as if the window is obscured by other windows,
+ * but no area outside of this region will be affected by drawing
+ * primitives.
+ * 
+ * Return value: a #GdkRegion. This must be freed with gdk_region_destroy()
+ *               when you are done.
+ **/
+GdkRegion *
+gdk_drawable_get_clip_region (GdkDrawable *drawable)
+{
+  g_return_val_if_fail (GDK_IS_DRAWABLE (drawable), NULL);
+
+  return GDK_DRAWABLE_GET_CLASS (drawable)->get_clip_region (drawable);
+}
+
+/**
+ * gdk_drawable_get_visible_region:
+ * @drawable: 
+ * 
+ * Computes the region of a drawable that is potentially visible.
+ * This does not necessarily take into account if the window is
+ * obscured by other windows, but no area outside of this region
+ * is visible.
+ * 
+ * Return value: a #GdkRegion. This must be freed with gdk_region_destroy()
+ *               when you are done.
+ **/
+GdkRegion *
+gdk_drawable_get_visible_region (GdkDrawable *drawable)
+{
+  g_return_val_if_fail (GDK_IS_DRAWABLE (drawable), NULL);
+
+  return GDK_DRAWABLE_GET_CLASS (drawable)->get_visible_region (drawable);
+}
+
+static GdkRegion *
+gdk_drawable_real_get_visible_region (GdkDrawable *drawable)
+{
+  GdkRectangle rect;
+
+  rect.x = 0;
+  rect.y = 0;
+
+  gdk_drawable_get_size (drawable, &rect.width, &rect.height);
+
+  return gdk_region_rectangle (&rect);
+}
