@@ -35,6 +35,8 @@
 #include <time.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/vt.h>
+#include <sys/kd.h>
 
 #include "gdk.h"
 
@@ -170,7 +172,7 @@ fb_modes_parse_mode (GScanner *scanner,
       switch (GPOINTER_TO_INT (scanner->value.v_symbol))
 	{
 	case FB_GEOMETRY:
-	  for (i=0;i<5;i++) {
+	  for (i = 0; i < 5;i++) {
 	    token = g_scanner_get_next_token (scanner);
 	    if (token != G_TOKEN_INT)
 	      {
@@ -182,7 +184,7 @@ fb_modes_parse_mode (GScanner *scanner,
 	  found_geometry = TRUE;
 	  break;
 	case FB_TIMINGS:
-	  for (i=0;i<7;i++) {
+	  for (i = 0; i < 7; i++) {
 	    token = g_scanner_get_next_token (scanner);
 	    if (token != G_TOKEN_INT)
 	      {
@@ -308,7 +310,7 @@ fb_modes_parse_mode (GScanner *scanner,
 
   if (strcmp (modename, specified_modename)== 0) {
     if (!found_geometry)
-      g_warning ("Geometry not specified\n");
+      g_warning ("Geometry not specified");
 
     if (found_geometry)
       {
@@ -320,7 +322,7 @@ fb_modes_parse_mode (GScanner *scanner,
       }
     
     if (!found_timings)
-      g_warning ("Timing not specified\n");
+      g_warning ("Timing not specified");
     
     if (found_timings)
       {
@@ -371,14 +373,14 @@ gdk_fb_setup_mode_from_name (struct fb_var_screeninfo *modeinfo,
   fd = open (filename, O_RDONLY);
   if (fd < 0)
     {
-      g_warning ("Cannot read %s\n", filename);
+      g_warning ("Cannot read %s", filename);
       return retval;
     }
   
   scanner = g_scanner_new ((GScannerConfig *) &fb_modes_scanner_config);
   scanner->input_name = filename;
 
-  for (i=0;i<sizeof(fb_modes_keywords)/sizeof(fb_modes_keywords[0]);i++)
+  for (i = 0; i < sizeof(fb_modes_keywords)/sizeof(fb_modes_keywords[0]); i++)
     g_scanner_add_symbol (scanner, fb_modes_keywords[i], GINT_TO_POINTER (i));
 
   g_scanner_input_file (scanner, fd);
@@ -390,7 +392,7 @@ gdk_fb_setup_mode_from_name (struct fb_var_screeninfo *modeinfo,
     result = fb_modes_parse_mode (scanner, modeinfo, modename);
       
     if (result < 0) {
-      g_warning ("parse error in %s at line %d\n", filename, scanner->line);
+      g_warning ("parse error in %s at line %d", filename, scanner->line);
       break;
     }
     if (result > 0)
@@ -413,15 +415,22 @@ gdk_fb_set_mode (GdkFBDisplay *display)
 {
   char *env, *end;
   int depth, height, width;
+  gboolean changed;
   
-  if (ioctl (display->fd, FBIOGET_VSCREENINFO, &display->modeinfo) < 0)
+  if (ioctl (display->fb_fd, FBIOGET_VSCREENINFO, &display->modeinfo) < 0)
     return -1;
+
+  display->orig_modeinfo = display->modeinfo;
+
+  changed = FALSE;
   
   env = getenv ("GDK_DISPLAY_MODE");
   if (env)
     {
-      if (!gdk_fb_setup_mode_from_name (&display->modeinfo, env))
-	g_warning ("Couldn't find mode named '%s'\n", env);
+      if (gdk_fb_setup_mode_from_name (&display->modeinfo, env))
+	changed = TRUE;
+      else
+	g_warning ("Couldn't find mode named '%s'", env);
     }
 
   env = getenv ("GDK_DISPLAY_DEPTH");
@@ -429,7 +438,10 @@ gdk_fb_set_mode (GdkFBDisplay *display)
     {
       depth = strtol (env, &end, 10);
       if (env != end)
-	display->modeinfo.bits_per_pixel = depth;
+	{
+	  changed = TRUE;
+	  display->modeinfo.bits_per_pixel = depth;
+	}
     }
   
   env = getenv ("GDK_DISPLAY_WIDTH");
@@ -438,6 +450,7 @@ gdk_fb_set_mode (GdkFBDisplay *display)
       width = strtol (env, &end, 10);
       if (env != end)
 	{
+	  changed = TRUE;
 	  display->modeinfo.xres = width;
 	  display->modeinfo.xres_virtual = width;
 	}
@@ -449,87 +462,184 @@ gdk_fb_set_mode (GdkFBDisplay *display)
       height = strtol (env, &end, 10);
       if (env != end)
 	{
+	  changed = TRUE;
 	  display->modeinfo.yres = height;
 	  display->modeinfo.yres_virtual = height;
 	}
     }
 
-  if (ioctl (display->fd, FBIOPUT_VSCREENINFO, &display->modeinfo) < 0)
+  if (changed &&
+      (ioctl (display->fb_fd, FBIOPUT_VSCREENINFO, &display->modeinfo) < 0))
     {
-      g_warning ("Couldn't set specified mode\n");
+      g_warning ("Couldn't set specified mode");
       return -1;
     }
   
-  if (ioctl (display->fd, FBIOGET_FSCREENINFO, &display->sinfo) < 0)
+  if (ioctl (display->fb_fd, FBIOGET_FSCREENINFO, &display->sinfo) < 0)
     {
-      g_warning ("Error getting fixed screen info\n");
+      g_warning ("Error getting fixed screen info");
       return -1;
     }
   return 0;
 }
 
 static GdkFBDisplay *
-gdk_fb_display_new (const char *filename)
+gdk_fb_display_new ()
 {
-  int fd, n;
-  GdkFBDisplay *retval;
+  GdkFBDisplay *display;
+  gchar *fb_filename;
+  struct vt_stat vs;
+  int vt, n;
+  gchar *s, *send;
+  char buf[32];
 
-  fd = open (filename, O_RDWR);
-  if (fd < 0)
-    return NULL;
+  display = g_new0 (GdkFBDisplay, 1);
 
-  retval = g_new0 (GdkFBDisplay, 1);
-  retval->fd = fd;
-
-  if (gdk_fb_set_mode (retval) < 0)
+  display->console_fd = open ("/dev/console", O_RDWR);
+  if (display->console_fd < 0)
     {
-      g_free (retval);
+      g_warning ("Can't open /dev/console: %s", strerror (display->console_fd));
+      g_free (display);
+      return NULL;
+    }
+  
+  ioctl (display->console_fd, VT_GETSTATE, &vs);
+  display->start_vt = vs.v_active;
+
+  vt = display->start_vt;
+  s = getenv("GDK_VT");
+  if (s)
+    {
+      if (g_strcasecmp ("new", s)==0)
+	{
+	  n = ioctl (display->console_fd, VT_OPENQRY, &vt);
+	  if (n < 0 || vt == -1)
+	    g_error("Cannot allocate new VT");
+	}
+      else
+	{
+	  vt = strtol (s, &send, 10);
+	  if (s==send)
+	    {
+	      g_warning ("Cannot parse GDK_VT");
+	      vt = display->start_vt;
+	    }
+	}
+      
+    }
+
+  display->vt = vt;
+  
+  /* Switch to the new VT */
+  if (vt != display->start_vt)
+    {
+      ioctl (display->console_fd, VT_ACTIVATE, vt);
+      ioctl (display->console_fd, VT_WAITACTIVE, vt);
+    }
+  
+  /* Open the tty */
+  g_snprintf (buf, sizeof(buf), "/dev/tty%d", vt);
+  display->tty_fd = open (buf, O_RDWR|O_NONBLOCK);
+  if (display->tty_fd < 0)
+    {
+      g_warning ("Can't open %s: %s", buf, strerror (display->tty_fd));
+      close (display->console_fd);
+      g_free (display);
       return NULL;
     }
 
-  ioctl (retval->fd, FBIOBLANK, 0);
+  /* Set controlling tty */
+  ioctl (0, TIOCNOTTY, 0);
+  ioctl (display->tty_fd, TIOCSCTTY, 0);
+  
+  fb_filename = gdk_get_display ();
+  display->fb_fd = open (fb_filename, O_RDWR);
+  if (display->fb_fd < 0)
+    {
+      g_warning ("Can't open %s: %s", fb_filename, strerror (display->fb_fd));
+      g_free (fb_filename);
+      close (display->tty_fd);
+      close (display->console_fd);
+      g_free (display);
+      return NULL;
+    }
+  g_free (fb_filename);
+
+  if (gdk_fb_set_mode (display) < 0)
+    {
+      close (display->fb_fd);
+      close (display->tty_fd);
+      close (display->console_fd);
+      g_free (display);
+      return NULL;
+    }
+
+  /* Disable normal text on the console */
+  ioctl (display->fb_fd, KDSETMODE, KD_GRAPHICS);
+
+  ioctl (display->fb_fd, FBIOBLANK, 0);
 
   /* We used to use sinfo.smem_len, but that seemed to be broken in many cases */
-  retval->fbmem = mmap (NULL,
-			retval->modeinfo.yres * retval->sinfo.line_length,
-			PROT_READ|PROT_WRITE,
-			MAP_SHARED,
-			fd,
-			0);
-  g_assert (retval->fbmem != MAP_FAILED);
+  display->fb_mmap = mmap (NULL,
+			   display->modeinfo.yres * display->sinfo.line_length,
+			   PROT_READ|PROT_WRITE,
+			   MAP_SHARED,
+			   display->fb_fd,
+			   0);
+  g_assert (display->fb_mmap != MAP_FAILED);
 
-  
-  if (retval->sinfo.visual == FB_VISUAL_PSEUDOCOLOR)
+  if (display->sinfo.visual == FB_VISUAL_TRUECOLOR)
     {
-      guint16 red[256], green[256], blue[256];
-      struct fb_cmap cmap;
-      for(n = 0; n < 16; n++)
-	red[n] = green[n] = blue[n] = n << 12;
-      for(n = 16; n < 256; n++)
-	red[n] = green[n] = blue[n] = n << 8;
-      cmap.red = red; cmap.green = green; cmap.blue = blue; cmap.len = 256; cmap.start = 0;
-      ioctl (fd, FBIOPUTCMAP, &cmap);
+      display->red_byte = display->modeinfo.red.offset >> 3;
+      display->green_byte = display->modeinfo.green.offset >> 3;
+      display->blue_byte = display->modeinfo.blue.offset >> 3;
     }
 
-  if (retval->sinfo.visual == FB_VISUAL_TRUECOLOR)
+#ifdef ENABLE_SHADOW_FB
+  if (_gdk_fb_screen_angle % 2 == 0)
     {
-      retval->red_byte = retval->modeinfo.red.offset >> 3;
-      retval->green_byte = retval->modeinfo.green.offset >> 3;
-      retval->blue_byte = retval->modeinfo.blue.offset >> 3;
+      display->fb_width = display->modeinfo.xres;
+      display->fb_height = display->modeinfo.yres;
+    } 
+  else
+    {
+      display->fb_width = display->modeinfo.yres;
+      display->fb_height = display->modeinfo.xres;
     }
+  display->fb_stride =
+    display->fb_width * (display->modeinfo.bits_per_pixel / 8);
+  display->fb_mem = g_malloc(display->fb_height * display->fb_stride);
+#else
+  display->fb_mem = display->fb_mmap;
+  display->fb_width = display->modeinfo.xres;
+  display->fb_height = display->modeinfo.yres;
+  display->fb_stride = display->sinfo.line_length;
+#endif
 
-  return retval;
+  return display;
 }
 
 static void
-gdk_fb_display_destroy (GdkFBDisplay *fbd)
+gdk_fb_display_destroy (GdkFBDisplay *display)
 {
-  munmap (fbd->fbmem, fbd->modeinfo.yres * fbd->sinfo.line_length);
-  close (fbd->fd);
-  g_free (fbd);
-}
+  /* Restore old framebuffer mode */
+  ioctl (display->fb_fd, FBIOPUT_VSCREENINFO, &display->orig_modeinfo);
+  
+  /* Enable normal text on the console */
+  ioctl (display->fb_fd, KDSETMODE, KD_TEXT);
+  
+  munmap (display->fb_mmap, display->modeinfo.yres * display->sinfo.line_length);
+  close (display->fb_fd);
 
-extern void keyboard_init(void);
+  ioctl (display->console_fd, VT_ACTIVATE, display->start_vt);
+  ioctl (display->console_fd, VT_WAITACTIVE, display->start_vt);
+  if (display->vt != display->start_vt)
+    ioctl (display->console_fd, VT_DISALLOCATE, display->vt);
+  
+  close (display->tty_fd);
+  close (display->console_fd);
+  g_free (display);
+}
 
 gboolean
 _gdk_windowing_init_check (int argc, char **argv)
@@ -537,16 +647,37 @@ _gdk_windowing_init_check (int argc, char **argv)
   if (gdk_initialized)
     return TRUE;
 
-  keyboard_init ();
-  gdk_display = gdk_fb_display_new ("/dev/fb");
+  /* Create new session and become session leader */
+  setsid();
+
+  gdk_display = gdk_fb_display_new ();
 
   if (!gdk_display)
     return FALSE;
 
-  gdk_fb_font_init ();
+  gdk_shadow_fb_init ();
+  
+  if (!gdk_fb_keyboard_open ())
+    {
+      g_warning ("Failed to initialize keyboard");
+      gdk_fb_display_destroy (gdk_display);
+      gdk_display = NULL;
+      return FALSE;
+    }
+  
+  if (!gdk_fb_mouse_open ())
+    {
+      g_warning ("Failed to initialize mouse");
+      gdk_fb_keyboard_close ();
+      gdk_fb_display_destroy (gdk_display);
+      gdk_display = NULL;
+      return FALSE;
+    }
 
   gdk_initialized = TRUE;
 
+  gdk_selection_property = gdk_atom_intern ("GDK_SELECTION", FALSE);
+  
   return TRUE;
 }
 
@@ -612,20 +743,30 @@ gdk_fb_pointer_grab (GdkWindow *	  window,
       gdk_pointer_ungrab (time);
     }
 
+  gdk_fb_window_send_crossing_events (NULL,
+				      window,
+				      GDK_CROSSING_GRAB);  
+
+  if (event_mask & GDK_BUTTON_MOTION_MASK)
+      event_mask |=
+	GDK_BUTTON1_MOTION_MASK |
+	GDK_BUTTON2_MOTION_MASK |
+	GDK_BUTTON3_MOTION_MASK;
+  
   _gdk_fb_pointer_implicit_grab = implicit_grab;
 
   _gdk_fb_pointer_grab_window = gdk_window_ref (window);
-  _gdk_fb_pointer_grab_window_events = owner_events ? NULL : _gdk_fb_pointer_grab_window;
+  _gdk_fb_pointer_grab_owner_events = owner_events;
 
   _gdk_fb_pointer_grab_confine = confine_to ? gdk_window_ref (confine_to) : NULL;
   _gdk_fb_pointer_grab_events = event_mask;
   _gdk_fb_pointer_grab_cursor = cursor ? gdk_cursor_ref (cursor) : NULL;
 
+
+  
   if (cursor)
     gdk_fb_cursor_reset ();
 
-  gdk_fb_window_visibility_crossing (window, TRUE, TRUE);
-  
   return GDK_GRAB_SUCCESS;
 }
 
@@ -654,7 +795,9 @@ void
 gdk_fb_pointer_ungrab (guint32 time, gboolean implicit_grab)
 {
   gboolean have_grab_cursor = _gdk_fb_pointer_grab_cursor && 1;
-
+  GdkWindow *mousewin;
+  GdkWindow *old_grab_window;
+  
   if (!_gdk_fb_pointer_grab_window)
     return;
 
@@ -672,14 +815,19 @@ gdk_fb_pointer_ungrab (guint32 time, gboolean implicit_grab)
   if (have_grab_cursor)
     gdk_fb_cursor_reset ();
 
-  gdk_fb_window_visibility_crossing (_gdk_fb_pointer_grab_window, FALSE, TRUE);
+  old_grab_window = _gdk_fb_pointer_grab_window;
   
-  if (_gdk_fb_pointer_grab_window)
-    gdk_window_unref (_gdk_fb_pointer_grab_window);
   _gdk_fb_pointer_grab_window = NULL;
-  _gdk_fb_pointer_grab_window_events = NULL;
 
   _gdk_fb_pointer_implicit_grab = FALSE;
+
+  mousewin = gdk_window_at_pointer (NULL, NULL);
+  gdk_fb_window_send_crossing_events (old_grab_window,
+				      mousewin,
+				      GDK_CROSSING_UNGRAB);
+  
+  if (old_grab_window)
+    gdk_window_unref (old_grab_window);
 }
 
 /*
@@ -734,8 +882,8 @@ gdk_keyboard_grab (GdkWindow *	   window,
   if (_gdk_fb_pointer_grab_window)
     gdk_keyboard_ungrab (time);
 
-  if (!owner_events)
-    _gdk_fb_keyboard_grab_window = gdk_window_ref (window);
+  _gdk_fb_keyboard_grab_window = gdk_window_ref (window);
+  _gdk_fb_keyboard_grab_owner_events = owner_events;
   
   return GDK_GRAB_SUCCESS;
 }
@@ -781,7 +929,7 @@ gdk_keyboard_ungrab (guint32 time)
 gint
 gdk_screen_width (void)
 {
-  return gdk_display->modeinfo.xres;
+  return gdk_display->fb_width;
 }
 
 /*
@@ -802,7 +950,7 @@ gdk_screen_width (void)
 gint
 gdk_screen_height (void)
 {
-  return gdk_display->modeinfo.yres;
+  return gdk_display->fb_height;
 }
 
 /*
@@ -890,12 +1038,14 @@ extern void keyboard_shutdown(void);
 void
 gdk_windowing_exit (void)
 {
+
+  gdk_fb_mouse_close ();
+  
+  gdk_fb_keyboard_close ();
+  
   gdk_fb_display_destroy (gdk_display);
+  
   gdk_display = NULL;
-
-  gdk_fb_font_fini ();
-
-  keyboard_shutdown ();
 }
 
 gchar*
@@ -913,214 +1063,320 @@ gdk_keyval_from_name (const gchar *keyval_name)
 gchar *
 gdk_get_display(void)
 {
-  return g_strdup ("/dev/fb0");
+  gchar *s;
+
+  s = getenv ("GDK_DISPLAY");
+  if (s==0)
+    s = "/dev/fb0";
+  
+  return g_strdup (s);
+}
+
+
+void
+gdk_beep (void)
+{
+  static int pitch = 600, duration = 100;
+  gulong arg;
+
+  /* Thank you XFree86 */
+  arg = ((1193190 / pitch) & 0xffff) |
+    (((unsigned long)duration) << 16);
+
+  ioctl (gdk_display->tty_fd, KDMKTONE, arg);
 }
 
 /* utils */
-GdkEvent *
-gdk_event_make (GdkWindow *window,
-		GdkEventType type,
-		gboolean append_to_queue)
+static const guint type_masks[] = {
+  GDK_SUBSTRUCTURE_MASK, /* GDK_DELETE		= 0, */
+  GDK_STRUCTURE_MASK, /* GDK_DESTROY		= 1, */
+  GDK_EXPOSURE_MASK, /* GDK_EXPOSE		= 2, */
+  GDK_POINTER_MOTION_MASK, /* GDK_MOTION_NOTIFY	= 3, */
+  GDK_BUTTON_PRESS_MASK, /* GDK_BUTTON_PRESS	= 4, */
+  GDK_BUTTON_PRESS_MASK, /* GDK_2BUTTON_PRESS	= 5, */
+  GDK_BUTTON_PRESS_MASK, /* GDK_3BUTTON_PRESS	= 6, */
+  GDK_BUTTON_RELEASE_MASK, /* GDK_BUTTON_RELEASE	= 7, */
+  GDK_KEY_PRESS_MASK, /* GDK_KEY_PRESS	= 8, */
+  GDK_KEY_RELEASE_MASK, /* GDK_KEY_RELEASE	= 9, */
+  GDK_ENTER_NOTIFY_MASK, /* GDK_ENTER_NOTIFY	= 10, */
+  GDK_LEAVE_NOTIFY_MASK, /* GDK_LEAVE_NOTIFY	= 11, */
+  GDK_FOCUS_CHANGE_MASK, /* GDK_FOCUS_CHANGE	= 12, */
+  GDK_STRUCTURE_MASK, /* GDK_CONFIGURE		= 13, */
+  GDK_VISIBILITY_NOTIFY_MASK, /* GDK_MAP		= 14, */
+  GDK_VISIBILITY_NOTIFY_MASK, /* GDK_UNMAP		= 15, */
+  GDK_PROPERTY_CHANGE_MASK, /* GDK_PROPERTY_NOTIFY	= 16, */
+  GDK_PROPERTY_CHANGE_MASK, /* GDK_SELECTION_CLEAR	= 17, */
+  GDK_PROPERTY_CHANGE_MASK, /* GDK_SELECTION_REQUEST = 18, */
+  GDK_PROPERTY_CHANGE_MASK, /* GDK_SELECTION_NOTIFY	= 19, */
+  GDK_PROXIMITY_IN_MASK, /* GDK_PROXIMITY_IN	= 20, */
+  GDK_PROXIMITY_OUT_MASK, /* GDK_PROXIMITY_OUT	= 21, */
+  GDK_ALL_EVENTS_MASK, /* GDK_DRAG_ENTER        = 22, */
+  GDK_ALL_EVENTS_MASK, /* GDK_DRAG_LEAVE        = 23, */
+  GDK_ALL_EVENTS_MASK, /* GDK_DRAG_MOTION       = 24, */
+  GDK_ALL_EVENTS_MASK, /* GDK_DRAG_STATUS       = 25, */
+  GDK_ALL_EVENTS_MASK, /* GDK_DROP_START        = 26, */
+  GDK_ALL_EVENTS_MASK, /* GDK_DROP_FINISHED     = 27, */
+  GDK_ALL_EVENTS_MASK, /* GDK_CLIENT_EVENT	= 28, */
+  GDK_VISIBILITY_NOTIFY_MASK, /* GDK_VISIBILITY_NOTIFY = 29, */
+  GDK_EXPOSURE_MASK, /* GDK_NO_EXPOSE		= 30, */
+  GDK_SCROLL_MASK /* GDK_SCROLL            = 31 */
+};
+
+GdkWindow *
+gdk_fb_other_event_window (GdkWindow *window,
+			   GdkEventType type)
+{
+  guint32 evmask;
+  GdkWindow *w;
+
+  w = window;
+  while (w != gdk_parent_root)
+    {
+      /* Huge hack, so that we don't propagate events to GtkWindow->frame */
+      if ((w != window) &&
+	  (GDK_WINDOW_P (w)->window_type != GDK_WINDOW_CHILD) &&
+	  (g_object_get_data (G_OBJECT (w), "gdk-window-child-handler")))
+	  break;
+	  
+      evmask = GDK_WINDOW_IMPL_FBDATA(window)->event_mask;
+
+      if (evmask & type_masks[type])
+	return w;
+      
+      w = gdk_window_get_parent (w);
+    }
+  
+  return NULL;
+}
+
+GdkWindow *
+gdk_fb_pointer_event_window (GdkWindow *window,
+			     GdkEventType type)
 {
-  static const guint type_masks[] = {
-    GDK_SUBSTRUCTURE_MASK, /* GDK_DELETE		= 0, */
-    GDK_STRUCTURE_MASK, /* GDK_DESTROY		= 1, */
-    GDK_EXPOSURE_MASK, /* GDK_EXPOSE		= 2, */
-    GDK_POINTER_MOTION_MASK, /* GDK_MOTION_NOTIFY	= 3, */
-    GDK_BUTTON_PRESS_MASK, /* GDK_BUTTON_PRESS	= 4, */
-    GDK_BUTTON_PRESS_MASK, /* GDK_2BUTTON_PRESS	= 5, */
-    GDK_BUTTON_PRESS_MASK, /* GDK_3BUTTON_PRESS	= 6, */
-    GDK_BUTTON_RELEASE_MASK, /* GDK_BUTTON_RELEASE	= 7, */
-    GDK_KEY_PRESS_MASK, /* GDK_KEY_PRESS	= 8, */
-    GDK_KEY_RELEASE_MASK, /* GDK_KEY_RELEASE	= 9, */
-    GDK_ENTER_NOTIFY_MASK, /* GDK_ENTER_NOTIFY	= 10, */
-    GDK_LEAVE_NOTIFY_MASK, /* GDK_LEAVE_NOTIFY	= 11, */
-    GDK_FOCUS_CHANGE_MASK, /* GDK_FOCUS_CHANGE	= 12, */
-    GDK_STRUCTURE_MASK, /* GDK_CONFIGURE		= 13, */
-    GDK_VISIBILITY_NOTIFY_MASK, /* GDK_MAP		= 14, */
-    GDK_VISIBILITY_NOTIFY_MASK, /* GDK_UNMAP		= 15, */
-    GDK_PROPERTY_CHANGE_MASK, /* GDK_PROPERTY_NOTIFY	= 16, */
-    GDK_PROPERTY_CHANGE_MASK, /* GDK_SELECTION_CLEAR	= 17, */
-    GDK_PROPERTY_CHANGE_MASK, /* GDK_SELECTION_REQUEST = 18, */
-    GDK_PROPERTY_CHANGE_MASK, /* GDK_SELECTION_NOTIFY	= 19, */
-    GDK_PROXIMITY_IN_MASK, /* GDK_PROXIMITY_IN	= 20, */
-    GDK_PROXIMITY_OUT_MASK, /* GDK_PROXIMITY_OUT	= 21, */
-    GDK_ALL_EVENTS_MASK, /* GDK_DRAG_ENTER        = 22, */
-    GDK_ALL_EVENTS_MASK, /* GDK_DRAG_LEAVE        = 23, */
-    GDK_ALL_EVENTS_MASK, /* GDK_DRAG_MOTION       = 24, */
-    GDK_ALL_EVENTS_MASK, /* GDK_DRAG_STATUS       = 25, */
-    GDK_ALL_EVENTS_MASK, /* GDK_DROP_START        = 26, */
-    GDK_ALL_EVENTS_MASK, /* GDK_DROP_FINISHED     = 27, */
-    GDK_ALL_EVENTS_MASK, /* GDK_CLIENT_EVENT	= 28, */
-    GDK_VISIBILITY_NOTIFY_MASK, /* GDK_VISIBILITY_NOTIFY = 29, */
-    GDK_EXPOSURE_MASK, /* GDK_NO_EXPOSE		= 30, */
-    GDK_SCROLL_MASK /* GDK_SCROLL            = 31 */
-  };
   guint evmask;
-
-  evmask = GDK_WINDOW_IMPL_FBDATA(window)->event_mask;
-
-  /* Bad hack to make sure that things work semi-properly with owner_events */
-  if (_gdk_fb_pointer_grab_window)
-    evmask |= _gdk_fb_pointer_grab_events;
-  if (_gdk_fb_keyboard_grab_window)
-    evmask |= _gdk_fb_keyboard_grab_events;
-
-  if (evmask & GDK_BUTTON_MOTION_MASK)
+  GdkModifierType mask;
+  GdkWindow *w;
+	  
+  gdk_fb_mouse_get_info (NULL, NULL, &mask);
+  
+  if (_gdk_fb_pointer_grab_window &&
+      !_gdk_fb_pointer_grab_owner_events)
     {
-      evmask |= GDK_BUTTON1_MOTION_MASK|GDK_BUTTON2_MOTION_MASK|GDK_BUTTON3_MOTION_MASK;
-    }
+      evmask = _gdk_fb_pointer_grab_events;
 
-  if (evmask & (GDK_BUTTON1_MOTION_MASK|GDK_BUTTON2_MOTION_MASK|GDK_BUTTON3_MOTION_MASK))
-    {
-      gint x, y;
-      GdkModifierType mask;
-
-      gdk_input_get_mouseinfo (&x, &y, &mask);
-
-      if (((mask & GDK_BUTTON1_MASK) && (evmask & GDK_BUTTON1_MOTION_MASK)) ||
-	  ((mask & GDK_BUTTON2_MASK) && (evmask & GDK_BUTTON2_MOTION_MASK)) ||
-	  ((mask & GDK_BUTTON3_MASK) && (evmask & GDK_BUTTON3_MOTION_MASK)))
-	evmask |= GDK_POINTER_MOTION_MASK;
-    }
-
-  if (evmask & type_masks[type])
-    {
-      GdkEvent *event = gdk_event_new ();
-#if 0
-      guint32 the_time = g_latest_time.tv_sec * 1000 + g_latest_time.tv_usec / 1000;
-#else
-      guint32 the_time;
-      GTimeVal gcurtime;
-
-	g_get_current_time (&gcurtime);
-	the_time = gcurtime.tv_sec * 1000 + gcurtime.tv_usec / 1000;
-#endif
-
-      event->any.type = type;
-      event->any.window = gdk_window_ref (window);
-      event->any.send_event = FALSE;
-      switch (type)
+      if (evmask & (GDK_BUTTON1_MOTION_MASK | GDK_BUTTON2_MOTION_MASK | GDK_BUTTON3_MOTION_MASK))
 	{
-	case GDK_MOTION_NOTIFY:
-	  event->motion.time = the_time;
-	  event->motion.axes = NULL;
-	  break;
-	case GDK_BUTTON_PRESS:
-	case GDK_2BUTTON_PRESS:
-	case GDK_3BUTTON_PRESS:
-	case GDK_BUTTON_RELEASE:
-	  event->button.time = the_time;
-	  event->button.axes = NULL;
-	  break;
-	case GDK_KEY_PRESS:
-	case GDK_KEY_RELEASE:
-	  event->key.time = the_time;
-	  break;
-	case GDK_ENTER_NOTIFY:
-	case GDK_LEAVE_NOTIFY:
-	  event->crossing.time = the_time;
-	  break;
-
-	case GDK_PROPERTY_NOTIFY:
-	  event->property.time = the_time;
-	  break;
-
-	case GDK_SELECTION_CLEAR:
-	case GDK_SELECTION_REQUEST:
-	case GDK_SELECTION_NOTIFY:
-	  event->selection.time = the_time;
-	  break;
-	case GDK_PROXIMITY_IN:
-	case GDK_PROXIMITY_OUT:
-	  event->proximity.time = the_time;
-	  break;
-	case GDK_DRAG_ENTER:
-	case GDK_DRAG_LEAVE:
-	case GDK_DRAG_MOTION:
-	case GDK_DRAG_STATUS:
-	case GDK_DROP_START:
-	case GDK_DROP_FINISHED:
-	  event->dnd.time = the_time;
-	  break;
-
-	case GDK_FOCUS_CHANGE:
-	case GDK_CONFIGURE:
-	case GDK_MAP:
-	case GDK_UNMAP:
-	case GDK_CLIENT_EVENT:
-	case GDK_VISIBILITY_NOTIFY:
-	case GDK_NO_EXPOSE:
-	case GDK_SCROLL:
-	case GDK_DELETE:
-	case GDK_DESTROY:
-	case GDK_EXPOSE:
-	default:
-	  break;
+	  if (((mask & GDK_BUTTON1_MASK) && (evmask & GDK_BUTTON1_MOTION_MASK)) ||
+	      ((mask & GDK_BUTTON2_MASK) && (evmask & GDK_BUTTON2_MOTION_MASK)) ||
+	      ((mask & GDK_BUTTON3_MASK) && (evmask & GDK_BUTTON3_MOTION_MASK)))
+	    evmask |= GDK_POINTER_MOTION_MASK;
 	}
 
-      if (append_to_queue)
-	gdk_event_queue_append (event);
+      if (evmask & type_masks[type])
+	return _gdk_fb_pointer_grab_window;
+      else
+	return NULL;
+    }
 
-      return event;
+  w = window;
+  while (w != gdk_parent_root)
+    {
+      /* Huge hack, so that we don't propagate events to GtkWindow->frame */
+      if ((w != window) &&
+	  (GDK_WINDOW_P (w)->window_type != GDK_WINDOW_CHILD) &&
+	  (g_object_get_data (G_OBJECT (w), "gdk-window-child-handler")))
+	  break;
+      
+      evmask = GDK_WINDOW_IMPL_FBDATA(window)->event_mask;
+
+      if (evmask & (GDK_BUTTON1_MOTION_MASK | GDK_BUTTON2_MOTION_MASK | GDK_BUTTON3_MOTION_MASK))
+	{
+	  if (((mask & GDK_BUTTON1_MASK) && (evmask & GDK_BUTTON1_MOTION_MASK)) ||
+	      ((mask & GDK_BUTTON2_MASK) && (evmask & GDK_BUTTON2_MOTION_MASK)) ||
+	      ((mask & GDK_BUTTON3_MASK) && (evmask & GDK_BUTTON3_MOTION_MASK)))
+	    evmask |= GDK_POINTER_MOTION_MASK;
+	}
+
+      if (evmask & type_masks[type])
+	return w;
+      
+      w = gdk_window_get_parent (w);
+    }
+
+  if (_gdk_fb_pointer_grab_window &&
+      _gdk_fb_pointer_grab_owner_events)
+    {
+      evmask = _gdk_fb_pointer_grab_events;
+      
+      if (evmask & (GDK_BUTTON1_MOTION_MASK | GDK_BUTTON2_MOTION_MASK | GDK_BUTTON3_MOTION_MASK))
+	{
+	  if (((mask & GDK_BUTTON1_MASK) && (evmask & GDK_BUTTON1_MOTION_MASK)) ||
+	      ((mask & GDK_BUTTON2_MASK) && (evmask & GDK_BUTTON2_MOTION_MASK)) ||
+	      ((mask & GDK_BUTTON3_MASK) && (evmask & GDK_BUTTON3_MOTION_MASK)))
+	    evmask |= GDK_POINTER_MOTION_MASK;
+	}
+      
+      if (evmask & type_masks[type])
+	return _gdk_fb_pointer_grab_window;
+    }
+  
+  return NULL;
+}
+
+GdkWindow *
+gdk_fb_keyboard_event_window (GdkWindow *window,
+			      GdkEventType type)
+{
+  guint32 evmask;
+  GdkWindow *w;
+  
+  if (_gdk_fb_keyboard_grab_window &&
+      !_gdk_fb_keyboard_grab_owner_events)
+    {
+      return _gdk_fb_keyboard_grab_window;
+    }
+  
+  w = window;
+  while (w != gdk_parent_root)
+    {
+      /* Huge hack, so that we don't propagate events to GtkWindow->frame */
+      if ((w != window) &&
+	  (GDK_WINDOW_P (w)->window_type != GDK_WINDOW_CHILD) &&
+	  (g_object_get_data (G_OBJECT (w), "gdk-window-child-handler")))
+	  break;
+	  
+      evmask = GDK_WINDOW_IMPL_FBDATA(window)->event_mask;
+
+      if (evmask & type_masks[type])
+	return w;
+      
+      w = gdk_window_get_parent (w);
+    }
+  
+  if (_gdk_fb_keyboard_grab_window &&
+      _gdk_fb_keyboard_grab_owner_events)
+    {
+      return _gdk_fb_keyboard_grab_window;
     }
 
   return NULL;
 }
 
-/* Debug hack. Call to find malloc area overwrites: */
-void CM (void)
+GdkEvent *
+gdk_event_make (GdkWindow *window,
+		GdkEventType type,
+		gboolean append_to_queue)
 {
-  static gpointer mymem = NULL;
-  gpointer arry[256];
-  int i;
-
-  return;
-
-  free (mymem);
-
-  for(i = 0; i < sizeof(arry)/sizeof(arry[0]); i++)
-    arry[i] = malloc (i+1);
-  for(i = 0; i < sizeof(arry)/sizeof(arry[0]); i++)
-    free (arry[i]);
-
-  mymem = malloc (256);
+  GdkEvent *event = gdk_event_new ();
+  guint32 the_time;
+  
+  the_time = gdk_fb_get_time ();
+  
+  event->any.type = type;
+  event->any.window = gdk_window_ref (window);
+  event->any.send_event = FALSE;
+  switch (type)
+    {
+    case GDK_MOTION_NOTIFY:
+      event->motion.time = the_time;
+      event->motion.axes = NULL;
+      break;
+    case GDK_BUTTON_PRESS:
+    case GDK_2BUTTON_PRESS:
+    case GDK_3BUTTON_PRESS:
+    case GDK_BUTTON_RELEASE:
+      event->button.time = the_time;
+      event->button.axes = NULL;
+      break;
+    case GDK_KEY_PRESS:
+    case GDK_KEY_RELEASE:
+      event->key.time = the_time;
+      break;
+    case GDK_ENTER_NOTIFY:
+    case GDK_LEAVE_NOTIFY:
+      event->crossing.time = the_time;
+      break;
+      
+    case GDK_PROPERTY_NOTIFY:
+      event->property.time = the_time;
+      break;
+      
+    case GDK_SELECTION_CLEAR:
+    case GDK_SELECTION_REQUEST:
+    case GDK_SELECTION_NOTIFY:
+      event->selection.time = the_time;
+      break;
+    case GDK_PROXIMITY_IN:
+    case GDK_PROXIMITY_OUT:
+      event->proximity.time = the_time;
+      break;
+    case GDK_DRAG_ENTER:
+    case GDK_DRAG_LEAVE:
+    case GDK_DRAG_MOTION:
+    case GDK_DRAG_STATUS:
+    case GDK_DROP_START:
+    case GDK_DROP_FINISHED:
+      event->dnd.time = the_time;
+      break;
+      
+    case GDK_FOCUS_CHANGE:
+    case GDK_CONFIGURE:
+    case GDK_MAP:
+    case GDK_UNMAP:
+    case GDK_CLIENT_EVENT:
+    case GDK_VISIBILITY_NOTIFY:
+    case GDK_NO_EXPOSE:
+    case GDK_SCROLL:
+    case GDK_DELETE:
+    case GDK_DESTROY:
+    case GDK_EXPOSE:
+    default:
+      break;
+    }
+  
+  if (append_to_queue)
+    gdk_event_queue_append (event);
+  
+  return event;
 }
 
-/* XXX badhack */
-typedef struct _GdkWindowPaint GdkWindowPaint;
-
-struct _GdkWindowPaint
+void
+gdk_fb_set_rotation (GdkFBAngle angle)
 {
-  GdkRegion *region;
-  GdkPixmap *pixmap;
-  gint x_offset;
-  gint y_offset;
-};
-
-void RP (GdkDrawable *d)
-{
-#if 0
-  if (GDK_DRAWABLE_TYPE(d) == GDK_DRAWABLE_PIXMAP)
+  if (angle == _gdk_fb_screen_angle)
+    return;
+  
+#ifdef ENABLE_SHADOW_FB
+  if (gdk_display)
     {
-      if (!GDK_PIXMAP_FBDATA(d)->no_free_mem)
+      gdk_shadow_fb_stop_updates ();
+
+      gdk_fb_cursor_hide ();
+      
+      _gdk_fb_screen_angle = angle;
+
+      if (angle % 2 == 0)
 	{
-	  guchar *oldmem = GDK_DRAWABLE_FBDATA(d)->mem;
-	  guint len = ((GDK_DRAWABLE_IMPL_FBDATA(d)->width * GDK_DRAWABLE_IMPL_FBDATA(d)->depth + 7) / 8) * GDK_DRAWABLE_IMPL_FBDATA(d)->height;
-	  GDK_DRAWABLE_IMPL_FBDATA(d)->mem = g_malloc(len);
-	  memcpy(GDK_DRAWABLE_IMPL_FBDATA(d)->mem, oldmem, len);
-	  g_free(oldmem);
+	  gdk_display->fb_width = gdk_display->modeinfo.xres;
+	  gdk_display->fb_height = gdk_display->modeinfo.yres;
+	} 
+      else
+	{
+	  gdk_display->fb_width = gdk_display->modeinfo.yres;
+	  gdk_display->fb_height = gdk_display->modeinfo.xres;
 	}
+      gdk_display->fb_stride =
+	gdk_display->fb_width * (gdk_display->modeinfo.bits_per_pixel / 8);
+      
+      gdk_fb_recompute_all();
+      gdk_fb_redraw_all ();
+      
+      gdk_fb_cursor_unhide ();
     }
   else
-    {
-      GSList *priv = GDK_WINDOW_P(d)->paint_stack;
-      for(; priv; priv = priv->next)
-	{
-	  GdkWindowPaint *p = priv->data;
-
-	  RP(p->pixmap);
-	}
-    }
+    _gdk_fb_screen_angle = angle;
+#else
+  g_warning ("Screen rotation without shadow fb not supported.");
 #endif
 }
+
