@@ -254,7 +254,7 @@ terms specified in this license.
 static int
 alloc_color(Colormap  colormap,
 	    XColor   *color,
-	    gulong   *pixelp)
+	    guint    *pixelp)
 {
   PALETTEENTRY entry, closeEntry;
   unsigned int i;
@@ -2150,6 +2150,42 @@ gdk_colormap_alloc_colors (GdkColormap *colormap,
   return nremaining;
 }
 
+void
+gdk_colormap_query_color (GdkColormap *colormap,
+			  gulong       pixel,
+			  GdkColor    *result)
+{
+  GdkVisual *visual;
+
+  g_return_if_fail (GDK_IS_COLORMAP (colormap));
+  
+  visual = gdk_colormap_get_visual (colormap);
+
+  switch (visual->type) {
+  case GDK_VISUAL_DIRECT_COLOR:
+  case GDK_VISUAL_TRUE_COLOR:
+    result->red = 65535. * (double)((pixel & visual->red_mask) >> visual->red_shift) / ((1 << visual->red_prec) - 1);
+    result->green = 65535. * (double)((pixel & visual->green_mask) >> visual->green_shift) / ((1 << visual->green_prec) - 1);
+    result->blue = 65535. * (double)((pixel & visual->blue_mask) >> visual->blue_shift) / ((1 << visual->blue_prec) - 1);
+    break;
+  case GDK_VISUAL_STATIC_GRAY:
+  case GDK_VISUAL_GRAYSCALE:
+    result->red = result->green = result->blue = 65535. * (double)pixel/((1<<visual->depth) - 1);
+    break;
+  case GDK_VISUAL_STATIC_COLOR:
+    g_assert_not_reached ();
+    break;
+  case GDK_VISUAL_PSEUDO_COLOR:
+    result->red = colormap->colors[pixel].red;
+    result->green = colormap->colors[pixel].green;
+    result->blue = colormap->colors[pixel].blue;
+    break;
+  default:
+    g_assert_not_reached ();
+    break;
+  }
+}
+
 gboolean
 gdk_color_change (GdkColormap *colormap,
 		  GdkColor    *color)
@@ -2272,7 +2308,7 @@ gdk_win32_color_to_string (const GdkColor *color)
 {
   static char buf[100];
 
-  sprintf (buf, "(%.04x,%.04x,%.04x):%.06lx",
+  sprintf (buf, "(%.04x,%.04x,%.04x):%.06x",
 	   color->red, color->green, color->blue, color->pixel);
 
   return buf;
