@@ -29,6 +29,8 @@
 #include "gdkvisual.h"
 #include "gdkprivate-win32.h"
 
+#include <stdlib.h>
+
 static void  gdk_visual_decompose_mask (gulong     mask,
 					gint      *shift,
 					gint      *prec);
@@ -65,6 +67,9 @@ gdk_visual_init (void)
 
   if (rastercaps & RC_PALETTE)
     {
+      if (!getenv ("GDK_WIN32_ENABLE_BROKEN_PSEUDOCOLOR_VISUAL"))
+	g_error ("Palettized display (%d-colour) mode not supported on Windows.",
+		 GetDeviceCaps (gdk_display_hdc, SIZEPALETTE));
       system_visual->visual.type = GDK_VISUAL_PSEUDO_COLOR;
       numcolors = GetDeviceCaps (gdk_display_hdc, NUMCOLORS);
       sizepalette = GetDeviceCaps (gdk_display_hdc, SIZEPALETTE);
