@@ -27,6 +27,7 @@
 
 #include <stdlib.h>
 
+#include "gdk.h" /* gdk_rectangle_intersect */
 #include "gdkevents.h"
 #include "gdkpixmap.h"
 #include "gdkwindow.h"
@@ -44,6 +45,7 @@ static void         gdk_window_impl_win32_set_colormap (GdkDrawable *drawable,
 static void         gdk_window_impl_win32_get_size     (GdkDrawable *drawable,
 							gint *width,
 							gint *height);
+static GdkRegion*  gdk_window_impl_win32_get_visible_region (GdkDrawable *drawable);
 static void gdk_window_impl_win32_init       (GdkWindowImplWin32      *window);
 static void gdk_window_impl_win32_class_init (GdkWindowImplWin32Class *klass);
 static void gdk_window_impl_win32_finalize   (GObject                 *object);
@@ -112,6 +114,10 @@ gdk_window_impl_win32_class_init (GdkWindowImplWin32Class *klass)
   drawable_class->set_colormap = gdk_window_impl_win32_set_colormap;
   drawable_class->get_colormap = gdk_window_impl_win32_get_colormap;
   drawable_class->get_size = gdk_window_impl_win32_get_size;
+
+  /* Visible and clip regions are the same */
+  drawable_class->get_clip_region = gdk_window_impl_win32_get_visible_region;
+  drawable_class->get_visible_region = gdk_window_impl_win32_get_visible_region;
 }
 
 static void
@@ -158,6 +164,7 @@ gdk_window_impl_win32_get_colormap (GdkDrawable *drawable)
       drawable_impl->colormap == NULL)
     {
       drawable_impl->colormap = gdk_colormap_get_system ();
+      gdk_colormap_ref (drawable_impl->colormap);
     }
   
   return drawable_impl->colormap;
@@ -179,6 +186,7 @@ gdk_window_impl_win32_set_colormap (GdkDrawable *drawable,
   GDK_DRAWABLE_GET_CLASS (draw_impl)->set_colormap (drawable, cmap);
   
   /* XXX */
+  g_print("gdk_window_impl_win32_set_colormap: XXX\n");
 }
 
 static void
@@ -194,6 +202,22 @@ gdk_window_impl_win32_get_size (GdkDrawable *drawable,
     *height = GDK_WINDOW_IMPL_WIN32 (drawable)->height;
 }
 
+static GdkRegion*
+gdk_window_impl_win32_get_visible_region (GdkDrawable *drawable)
+{
+  GdkWindowImplWin32 *impl = GDK_WINDOW_IMPL_WIN32 (drawable);
+  GdkRectangle result_rect;
+
+  result_rect.x = 0;
+  result_rect.y = 0;
+  result_rect.width = impl->width;
+  result_rect.height = impl->height;
+
+  gdk_rectangle_intersect (&result_rect, &impl->position_info.clip_rect, &result_rect);
+
+  return gdk_region_rectangle (&result_rect);
+}
+
 void
 _gdk_windowing_window_init (void)
 {
@@ -305,10 +329,15 @@ RegisterGdkClass (GdkWindowType wtype)
   wcl.hIconSm = 0;
 
   /* initialize once per class */
+  /*
+   * HB: Setting the background brush leads to flicker, because we
+   * don't get asked how to clear the background. This is not what
+   * we want, at least not for input_only windows ...
+   */
 #define ONCE_PER_CLASS() \
   wcl.hIcon = CopyIcon (hAppIcon); \
   wcl.hIconSm = CopyIcon (hAppIcon); \
-  wcl.hbrBackground = CreateSolidBrush (RGB (0,0,0)); \
+  wcl.hbrBackground = NULL; /* CreateSolidBrush (RGB (0,0,0)); */ \
   wcl.hCursor = LoadCursor (NULL, IDC_ARROW); 
   
   switch (wtype)
@@ -393,9 +422,6 @@ gdk_window_new (GdkWindow     *parent,
   int width, height;
   int x, y;
   char *title;
-  gint titlelen;
-  wchar_t *wctitle;
-  gint wlen;
   char *mbtitle;
 
   g_return_val_if_fail (attributes != NULL, NULL);
@@ -405,6 +431,14 @@ gdk_window_new (GdkWindow     *parent,
 
   g_return_val_if_fail (GDK_IS_WINDOW (parent), NULL);
   
+  GDK_NOTE (MISC,
+	    g_print ("gdk_window_new: %s\n",
+		     (attributes->window_type == GDK_WINDOW_TOPLEVEL ? "TOPLEVEL" :
+		      (attributes->window_type == GDK_WINDOW_CHILD ? "CHILD" :
+		       (attributes->window_type == GDK_WINDOW_DIALOG ? "DIALOG" :
+			(attributes->window_type == GDK_WINDOW_TEMP ? "TEMP" :
+			 "???"))))));
+
   parent_private = (GdkWindowObject*) parent;
   if (GDK_WINDOW_DESTROYED (parent))
     return NULL;
@@ -452,7 +486,7 @@ gdk_window_new (GdkWindow     *parent,
     title = attributes->title;
   else
     title = g_get_prgname ();
-  if (!title)
+  if (!title || !*title)
     title = "GDK client window";
 
   impl->event_mask = GDK_STRUCTURE_MASK | attributes->event_mask;
@@ -478,14 +512,16 @@ gdk_window_new (GdkWindow     *parent,
 	{
 	  if ((((GdkVisualPrivate*)gdk_visual_get_system ())->xvisual) == xvisual)
             {
-              draw_impl->colormap =
-                gdk_colormap_get_system ();
+              draw_impl->colormap = gdk_colormap_get_system ();
               gdk_colormap_ref (draw_impl->colormap);
+	      GDK_NOTE (MISC, g_print ("...using system colormap %p\n",
+				       draw_impl->colormap));
             }
 	  else
             {
-              draw_impl->colormap =
-                gdk_colormap_new (visual, FALSE);
+              draw_impl->colormap = gdk_colormap_new (visual, FALSE);
+	      GDK_NOTE (MISC, g_print ("...using new colormap %p\n",
+				       draw_impl->colormap));
             }
 	}
     }
@@ -495,6 +531,7 @@ gdk_window_new (GdkWindow     *parent,
       private->depth = 0;
       private->input_only = TRUE;
       draw_impl->colormap = NULL;
+      GDK_NOTE (MISC, g_print ("...GDK_INPUT_ONLY, NULL colormap\n"));
     }
 
   if (parent_private)
@@ -563,13 +600,7 @@ gdk_window_new (GdkWindow     *parent,
       height = impl->height;
     }
 
-  titlelen = strlen (title);
-  wctitle = g_new (wchar_t, titlelen + 1);
-  mbtitle = g_new (char, 3*titlelen + 1);
-  wlen = gdk_nmbstowchar_ts (wctitle, title, titlelen, titlelen);
-  wctitle[wlen] = 0;
-  WideCharToMultiByte (GetACP (), 0, wctitle, -1,
-		       mbtitle, 3*titlelen, NULL, NULL);
+  mbtitle = g_locale_from_utf8 (title, -1, NULL, NULL, NULL);
   
 #ifdef WITHOUT_WM_CREATE
   draw_impl->handle = CreateWindowEx (dwExStyle,
@@ -622,13 +653,8 @@ gdk_window_new (GdkWindow     *parent,
 #endif
 
   GDK_NOTE (MISC,
-	    g_print ("gdk_window_new: %s %s %dx%d@+%d+%d %#x = %#x\n"
-		     "...locale %#x codepage %d\n",
-		     (private->window_type == GDK_WINDOW_TOPLEVEL ? "TOPLEVEL" :
-		      (private->window_type == GDK_WINDOW_CHILD ? "CHILD" :
-		       (private->window_type == GDK_WINDOW_DIALOG ? "DIALOG" :
-			(private->window_type == GDK_WINDOW_TEMP ? "TEMP" :
-			 "???")))),
+	    g_print ("... \"%s\" %dx%d@+%d+%d %#x = %#x\n"
+		     "... locale %#x codepage %d\n",
 		     mbtitle,
 		     width, height, (x == CW_USEDEFAULT ? -9999 : x), y, 
 		     (guint) hparent,
@@ -637,7 +663,6 @@ gdk_window_new (GdkWindow     *parent,
 		     (guint) impl->charset_info.ciACP));
 
   g_free (mbtitle);
-  g_free (wctitle);
 
   if (draw_impl->handle == NULL)
     {
@@ -698,7 +723,10 @@ gdk_window_foreign_new (GdkNativeWindow anid)
   impl->height = rect.bottom - rect.top;
   private->window_type = GDK_WINDOW_FOREIGN;
   private->destroyed = FALSE;
-  private->mapped = IsWindowVisible ((HWND) anid);
+  if (IsWindowVisible ((HWND) anid))
+    private->state &= (~GDK_WINDOW_STATE_WITHDRAWN);
+  else
+    private->state |= GDK_WINDOW_STATE_WITHDRAWN;
   private->depth = gdk_visual_get_system ()->depth;
 
   gdk_drawable_ref (window);
@@ -784,7 +812,7 @@ gdk_window_show (GdkWindow *window)
       GDK_NOTE (MISC, g_print ("gdk_window_show: %#x\n",
 			       (guint) GDK_WINDOW_HWND (window)));
 
-      private->mapped = TRUE;
+      private->state &= (~GDK_WINDOW_STATE_WITHDRAWN);
       if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_TEMP)
 	{
 	  ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNOACTIVATE);
@@ -797,13 +825,24 @@ gdk_window_show (GdkWindow *window)
 	}
       else
 	{
-	  ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNORMAL);
-	  ShowWindow (GDK_WINDOW_HWND (window), SW_RESTORE);
-	  SetForegroundWindow (GDK_WINDOW_HWND (window));
-	  BringWindowToTop (GDK_WINDOW_HWND (window));
+          if (GetWindowLong (GDK_WINDOW_HWND (window), GWL_EXSTYLE) & WS_EX_TRANSPARENT)
+	    {
+	      SetWindowPos(GDK_WINDOW_HWND (window), HWND_TOP, 0, 0, 0, 0,
+			   SWP_SHOWWINDOW | SWP_NOREDRAW | SWP_NOMOVE | SWP_NOSIZE);
+	    }
+          else
+            {
+	      GdkWindow *parent = GDK_WINDOW (private->parent);
+
+	      ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNORMAL);
+	      ShowWindow (GDK_WINDOW_HWND (window), SW_RESTORE);
+              if (parent == gdk_parent_root)
+                SetForegroundWindow (GDK_WINDOW_HWND (window));
+	      BringWindowToTop (GDK_WINDOW_HWND (window));
 #if 0
-	  ShowOwnedPopups (GDK_WINDOW_HWND (window), TRUE);
+	      ShowOwnedPopups (GDK_WINDOW_HWND (window), TRUE);
 #endif
+	    }
 	}
     }
 }
@@ -821,11 +860,19 @@ gdk_window_hide (GdkWindow *window)
       GDK_NOTE (MISC, g_print ("gdk_window_hide: %#x\n",
 			       (guint) GDK_WINDOW_HWND (window)));
 
-      private->mapped = FALSE;
+      private->state |= GDK_WINDOW_STATE_WITHDRAWN;
       if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_TOPLEVEL)
 	ShowOwnedPopups (GDK_WINDOW_HWND (window), FALSE);
 
-      ShowWindow (GDK_WINDOW_HWND (window), SW_HIDE);
+      if (GetWindowLong (GDK_WINDOW_HWND (window), GWL_EXSTYLE) & WS_EX_TRANSPARENT)
+	{
+	  SetWindowPos(GDK_WINDOW_HWND (window), HWND_BOTTOM, 0, 0, 0, 0,
+		       SWP_HIDEWINDOW | SWP_NOREDRAW | SWP_NOZORDER | SWP_NOMOVE | SWP_NOSIZE);
+	}
+      else
+	{
+	  ShowWindow (GDK_WINDOW_HWND (window), SW_HIDE);
+	}
     }
 }
 
@@ -1313,9 +1360,13 @@ gdk_window_set_geometry_hints (GdkWindow      *window,
       rect.bottom = geometry->max_height;
       dwStyle = GetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE);
       dwExStyle = GetWindowLong (GDK_WINDOW_HWND (window), GWL_EXSTYLE);
-      AdjustWindowRectEx (&rect, dwStyle, FALSE, dwExStyle);
+      /* HB: dont' know why AdjustWindowRectEx is called here, ... */
+      SafeAdjustWindowRectEx (&rect, dwStyle, FALSE, dwExStyle);
       impl->hint_max_width = rect.right - rect.left;
       impl->hint_max_height = rect.bottom - rect.top;
+      /* ... but negative sizes are always wrong */
+      if (impl->hint_max_width < 0) impl->hint_max_width = G_MAXSHORT;
+      if (impl->hint_max_height < 0) impl->hint_max_height = G_MAXSHORT;
 
       /* Again, check if the window is too large currently. */
       GetClientRect (GDK_WINDOW_HWND (window), &rect);
@@ -1372,10 +1423,7 @@ void
 gdk_window_set_title (GdkWindow   *window,
 		      const gchar *title)
 {
-  gint titlelen;
-  wchar_t *wcstr;
-  gint wlen;
-  char *mbstr;
+  char *mbtitle;
 
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
@@ -1393,19 +1441,11 @@ gdk_window_set_title (GdkWindow   *window,
       /* As the title is in UTF-8 we must translate it
        * to the system codepage.
        */
-      titlelen = strlen (title);
-      wcstr = g_new (wchar_t, titlelen + 1);
-      mbstr = g_new (char, 3*titlelen + 1);
-      wlen = gdk_nmbstowchar_ts (wcstr, title, titlelen, titlelen);
-      wcstr[wlen] = 0;
-      WideCharToMultiByte (GetACP (), 0, wcstr, -1,
-			   mbstr, 3*titlelen, NULL, NULL);
-
-      if (!SetWindowText (GDK_WINDOW_HWND (window), mbstr))
+      mbtitle = g_locale_from_utf8 (title, -1, NULL, NULL, NULL);
+      if (!SetWindowText (GDK_WINDOW_HWND (window), mbtitle))
 	WIN32_API_FAILED ("SetWindowText");
 
-      g_free (mbstr);
-      g_free (wcstr);
+      g_free (mbtitle);
     }
 }
 
@@ -1500,6 +1540,7 @@ gdk_window_set_cursor (GdkWindow *window,
   GdkWindowImplWin32 *impl;
   GdkCursorPrivate *cursor_private;
   HCURSOR hcursor;
+  HCURSOR hprevcursor;
   POINT pt;
   
   g_return_if_fail (window != NULL);
@@ -1518,15 +1559,10 @@ gdk_window_set_cursor (GdkWindow *window,
       GDK_NOTE (MISC, g_print ("gdk_window_set_cursor: %#x %#x\n",
 			       (guint) GDK_WINDOW_HWND (window),
 			       (guint) hcursor));
-      if (impl->hcursor != NULL)
-	{
-	  GDK_NOTE (MISC, g_print ("...DestroyCursor (%#x)\n",
-				   (guint) impl->hcursor));
-	  
-	  DestroyCursor (impl->hcursor);
-	  impl->hcursor = NULL;
-	}
-      if (hcursor != NULL)
+      hprevcursor = impl->hcursor;
+      if (hcursor == NULL)
+	impl->hcursor = NULL;
+      else
 	{
 	  /* We must copy the cursor as it is OK to destroy the GdkCursor
 	   * while still in use for some window. See for instance
@@ -1538,9 +1574,17 @@ gdk_window_set_cursor (GdkWindow *window,
 	  GDK_NOTE (MISC, g_print ("...CopyCursor (%#x) = %#x\n",
 				   (guint) hcursor, (guint) impl->hcursor));
 
-	  GetCursorPos (&pt);
-	  if (ChildWindowFromPoint (GDK_WINDOW_HWND (window), pt) == GDK_WINDOW_HWND (window))
+	  if (hprevcursor != NULL && GetCursor () == hprevcursor)
 	    SetCursor (impl->hcursor);
+
+	  if (hprevcursor != NULL)
+	    {
+	      GDK_NOTE (MISC, g_print ("...DestroyCursor (%#x)\n",
+				       (guint) hprevcursor));
+	  
+	      if (!DestroyCursor (hprevcursor))
+		WIN32_API_FAILED ("DestroyCursor");
+	    }
 	}
     }
 }
@@ -1661,6 +1705,39 @@ gdk_window_get_root_origin (GdkWindow *window,
 			   pt.x, pt.y));
 }
 
+void
+gdk_window_get_frame_extents (GdkWindow    *window,
+                              GdkRectangle *rect)
+{
+  GdkWindowObject *private;
+  HWND hwnd;
+  RECT r;
+
+  rect->x = 0;
+  rect->y = 0;
+  rect->width = 1;
+  rect->height = 1;
+  
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  hwnd = GDK_WINDOW_HWND (window);
+  /* find the frame window */
+  while (HWND_DESKTOP != GetParent (hwnd))
+    {
+      hwnd = GetParent (hwnd);
+      g_return_if_fail (NULL != hwnd);
+    }
+
+  if (GetWindowRect (hwnd, &r))
+    WIN32_API_FAILED ("GetWindowRect");
+
+  rect->x = r.left;
+  rect->y = r.right;
+  rect->width = r.right - r.left;
+  rect->height = r.bottom - r.top;
+}
+
 GdkWindow*
 gdk_window_get_pointer (GdkWindow       *window,
 			gint            *x,
@@ -2185,3 +2262,247 @@ gdk_window_set_static_gravities (GdkWindow *window,
   
   return TRUE;
 }
+
+/*
+ * Setting window states
+ */
+void
+gdk_window_iconify (GdkWindow *window)
+{
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  if (GDK_WINDOW_IS_MAPPED (window))
+    {
+      ShowWindow (GDK_WINDOW_HWND (window), SW_MINIMIZE);
+    }
+  else
+    {
+      /* Flip our client side flag, the real work happens on map. */
+      gdk_synthesize_window_state (window,
+                                   0,
+                                   GDK_WINDOW_STATE_ICONIFIED);
+    }
+}
+
+void
+gdk_window_deiconify (GdkWindow *window)
+{
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  if (GDK_WINDOW_IS_MAPPED (window))
+    {  
+      gdk_window_show (window);
+    }
+  else
+    {
+      /* Flip our client side flag, the real work happens on map. */
+      gdk_synthesize_window_state (window,
+                                   GDK_WINDOW_STATE_ICONIFIED,
+                                   0);
+    }
+}
+
+void
+gdk_window_stick (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  if (GDK_WINDOW_IS_MAPPED (window))
+    {
+      /* "stick" means stick to all desktops _and_ do not scroll with the
+       * viewport. i.e. glue to the monitor glass in all cases.
+       */
+      g_warning ("gdk_window_stick (0x%X) ???", GDK_WINDOW_HWND (window));
+    }
+  else
+    {
+      /* Flip our client side flag, the real work happens on map. */
+      gdk_synthesize_window_state (window,
+                                   0,
+                                   GDK_WINDOW_STATE_STICKY);
+    }
+}
+
+void
+gdk_window_unstick (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  if (GDK_WINDOW_IS_MAPPED (window))
+    {
+      g_warning ("gdk_window_unstick (0x%X) ???", GDK_WINDOW_HWND (window));
+    }
+  else
+    {
+      /* Flip our client side flag, the real work happens on map. */
+      gdk_synthesize_window_state (window,
+                                   GDK_WINDOW_STATE_STICKY,
+                                   0);
+
+    }
+}
+
+void
+gdk_window_maximize (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  if (GDK_WINDOW_IS_MAPPED (window))
+    ShowWindow (GDK_WINDOW_HWND (window), SW_MAXIMIZE);
+  else
+    gdk_synthesize_window_state (window,
+				 0,
+				 GDK_WINDOW_STATE_MAXIMIZED);
+}
+
+void
+gdk_window_unmaximize (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  if (GDK_WINDOW_IS_MAPPED (window))
+    ShowWindow (GDK_WINDOW_HWND (window), SW_RESTORE);
+  else
+    gdk_synthesize_window_state (window,
+				 GDK_WINDOW_STATE_MAXIMIZED,
+				 0);
+}
+
+void
+gdk_window_focus (GdkWindow *window,
+                  guint32    timestamp)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+  
+  ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNORMAL);
+}
+
+void
+gdk_window_set_modal_hint (GdkWindow *window,
+			   gboolean   modal)
+{
+  GdkWindowObject *private;
+
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  private = (GdkWindowObject*) window;
+
+  private->modal_hint = modal;
+
+  if (GDK_WINDOW_IS_MAPPED (window))
+    if (!SetWindowPos (GDK_WINDOW_HWND (window), HWND_TOPMOST,
+                           0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE))
+      WIN32_API_FAILED ("SetWindowPos");
+}
+
+void
+gdk_window_set_type_hint (GdkWindow        *window,
+			  GdkWindowTypeHint hint)
+{
+  GdkAtom atom;
+  
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  switch (hint)
+    {
+    case GDK_WINDOW_TYPE_HINT_DIALOG:
+      atom = gdk_atom_intern ("_NET_WM_WINDOW_TYPE_DIALOG", FALSE);
+      break;
+    case GDK_WINDOW_TYPE_HINT_MENU:
+      atom = gdk_atom_intern ("_NET_WM_WINDOW_TYPE_MENU", FALSE);
+      break;
+    case GDK_WINDOW_TYPE_HINT_TOOLBAR:
+      atom = gdk_atom_intern ("_NET_WM_WINDOW_TYPE_TOOLBAR", FALSE);
+      break;
+    default:
+      g_warning ("Unknown hint %d passed to gdk_window_set_type_hint", hint);
+      /* Fall thru */
+    case GDK_WINDOW_TYPE_HINT_NORMAL:
+      atom = gdk_atom_intern ("_NET_WM_WINDOW_TYPE_NORMAL", FALSE);
+      break;
+    }
+  /*
+   * XXX ???
+   */
+  GDK_NOTE (MISC,
+            g_print ("gdk_window_set_type_hint (0x%0X)\n",
+                     GDK_WINDOW_HWND (window)));
+}
+
+void
+gdk_window_shape_combine_region (GdkWindow *window,
+                                 GdkRegion *shape_region,
+                                 gint       offset_x,
+                                 gint       offset_y)
+{
+  gint xoffset, yoffset;
+  
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  /* XXX: even on X implemented conditional ... */  
+}
+
+void
+gdk_window_begin_resize_drag (GdkWindow     *window,
+                              GdkWindowEdge  edge,
+                              gint           button,
+                              gint           root_x,
+                              gint           root_y,
+                              guint32        timestamp)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  /* XXX: isn't all this default on win32 ... */  
+}
+
+void
+gdk_window_begin_move_drag (GdkWindow *window,
+                            gint       button,
+                            gint       root_x,
+                            gint       root_y,
+                            guint32    timestamp)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  /* XXX: isn't all this default on win32 ... */  
+}
\ No newline at end of file
