@@ -31,10 +31,6 @@ static void gtk_bin_class_init (GtkBinClass    *klass);
 static void gtk_bin_init       (GtkBin         *bin);
 static void gtk_bin_map        (GtkWidget      *widget);
 static void gtk_bin_unmap      (GtkWidget      *widget);
-static void gtk_bin_draw       (GtkWidget      *widget,
-			        GdkRectangle   *area);
-static gint gtk_bin_expose     (GtkWidget      *widget,
-			        GdkEventExpose *event);
 static void gtk_bin_add        (GtkContainer   *container,
 			        GtkWidget      *widget);
 static void gtk_bin_remove     (GtkContainer   *container,
@@ -89,8 +85,6 @@ gtk_bin_class_init (GtkBinClass *class)
 
   widget_class->map = gtk_bin_map;
   widget_class->unmap = gtk_bin_unmap;
-  widget_class->draw = gtk_bin_draw;
-  widget_class->expose_event = gtk_bin_expose;
 
   container_class->add = gtk_bin_add;
   container_class->remove = gtk_bin_remove;
@@ -154,52 +148,6 @@ gtk_bin_unmap (GtkWidget *widget)
     gtk_widget_unmap (bin->child);
 }
 
-static void
-gtk_bin_draw (GtkWidget    *widget,
-	      GdkRectangle *area)
-{
-  GtkBin *bin;
-  GdkRectangle child_area;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_BIN (widget));
-
-  bin = GTK_BIN (widget);
-
-  if (GTK_WIDGET_DRAWABLE (bin))
-    {
-      if (bin->child && GTK_WIDGET_DRAWABLE (bin->child) &&
-	  gtk_widget_intersect (bin->child, area, &child_area))
-	gtk_widget_draw (bin->child, &child_area);
-    }
-}
-
-static gint
-gtk_bin_expose (GtkWidget      *widget,
-		GdkEventExpose *event)
-{
-  GtkBin *bin;
-  GdkEventExpose child_event;
-
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_BIN (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      bin = GTK_BIN (widget);
-
-      child_event = *event;
-      if (bin->child && GTK_WIDGET_DRAWABLE (bin->child) &&
-	  GTK_WIDGET_NO_WINDOW (bin->child) &&
-	  gtk_widget_intersect (bin->child, &event->area, &child_event.area))
-	gtk_widget_event (bin->child, (GdkEvent*) &child_event);
-    }
-
-  return FALSE;
-}
-
-
 static void
 gtk_bin_add (GtkContainer *container,
 	     GtkWidget    *child)
@@ -212,7 +160,18 @@ gtk_bin_add (GtkContainer *container,
   g_return_if_fail (GTK_IS_WIDGET (child));
 
   bin = GTK_BIN (container);
-  g_return_if_fail (bin->child == NULL);
+
+  if (bin->child != NULL)
+    {
+      g_warning ("Attempting to add a widget with type %s to a %s, "
+                 "but as a GtkBin subclass a %s can only contain one widget at a time; "
+                 "it already contains a widget of type %s",
+                 g_type_name (G_OBJECT_TYPE (child)),
+                 g_type_name (G_OBJECT_TYPE (bin)),
+                 g_type_name (G_OBJECT_TYPE (bin)),
+                 g_type_name (G_OBJECT_TYPE (bin->child)));
+      return;
+    }
 
   gtk_widget_set_parent (child, GTK_WIDGET (bin));
   bin->child = child;
