@@ -18,7 +18,7 @@
  */
 
 /*
- * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * Modified by the GTK+ Team and others 1997-2001.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
@@ -27,6 +27,7 @@
 #include "gtkcheckmenuitem.h"
 #include "gtkaccellabel.h"
 #include "gtksignal.h"
+#include "gtkintl.h"
 
 #define CHECK_TOGGLE_SIZE 12
 
@@ -35,20 +36,31 @@ enum {
   LAST_SIGNAL
 };
 
+enum {
+  PROP_0,
+  PROP_ACTIVE,
+  PROP_INCONSISTENT
+};
 
 static void gtk_check_menu_item_class_init           (GtkCheckMenuItemClass *klass);
 static void gtk_check_menu_item_init                 (GtkCheckMenuItem      *check_menu_item);
-static void gtk_check_menu_item_draw                 (GtkWidget             *widget,
-						      GdkRectangle          *area);
 static gint gtk_check_menu_item_expose               (GtkWidget             *widget,
 						      GdkEventExpose        *event);
 static void gtk_check_menu_item_activate             (GtkMenuItem           *menu_item);
 static void gtk_check_menu_item_toggle_size_request  (GtkMenuItem           *menu_item,
-						      guint16               *requisition);
+						      gint                  *requisition);
 static void gtk_check_menu_item_draw_indicator       (GtkCheckMenuItem      *check_menu_item,
 						      GdkRectangle          *area);
 static void gtk_real_check_menu_item_draw_indicator  (GtkCheckMenuItem      *check_menu_item,
 						      GdkRectangle          *area);
+static void gtk_check_menu_item_set_property (GObject         *object,
+					      guint            prop_id,
+					      const GValue    *value,
+					      GParamSpec      *pspec);
+static void gtk_check_menu_item_get_property (GObject         *object,
+					      guint            prop_id,
+					      GValue          *value,
+					      GParamSpec      *pspec);
 
 
 static GtkMenuItemClass *parent_class = NULL;
@@ -92,8 +104,26 @@ gtk_check_menu_item_class_init (GtkCheckMenuItemClass *klass)
   menu_item_class = (GtkMenuItemClass*) klass;
   
   parent_class = gtk_type_class (GTK_TYPE_MENU_ITEM);
+
+  G_OBJECT_CLASS(object_class)->set_property = gtk_check_menu_item_set_property;
+  G_OBJECT_CLASS(object_class)->get_property = gtk_check_menu_item_get_property;
+
+  g_object_class_install_property (G_OBJECT_CLASS (object_class),
+                                   PROP_ACTIVE,
+                                   g_param_spec_boolean ("active",
+                                                         _("Active"),
+                                                         _("Whether the menu item is checked."),
+                                                         FALSE,
+                                                         G_PARAM_READABLE | G_PARAM_WRITABLE));
+  
+  g_object_class_install_property (G_OBJECT_CLASS (object_class),
+                                   PROP_INCONSISTENT,
+                                   g_param_spec_boolean ("inconsistent",
+                                                         _("Inconsistent"),
+                                                         _("Whether to display an \"inconsistent\" state."),
+                                                         FALSE,
+                                                         G_PARAM_READABLE | G_PARAM_WRITABLE));
   
-  widget_class->draw = gtk_check_menu_item_draw;
   widget_class->expose_event = gtk_check_menu_item_expose;
   
   menu_item_class->activate = gtk_check_menu_item_activate;
@@ -110,7 +140,6 @@ gtk_check_menu_item_class_init (GtkCheckMenuItemClass *klass)
                     GTK_SIGNAL_OFFSET (GtkCheckMenuItemClass, toggled),
                     gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
-  gtk_object_class_add_signals (object_class, check_menu_item_signals, LAST_SIGNAL);
 }
 
 GtkWidget*
@@ -146,12 +175,15 @@ gtk_check_menu_item_set_active (GtkCheckMenuItem *check_menu_item,
   is_active = is_active != 0;
 
   if (check_menu_item->active != is_active)
-    gtk_menu_item_activate (GTK_MENU_ITEM (check_menu_item));
+     {
+	gtk_menu_item_activate (GTK_MENU_ITEM (check_menu_item));
+	g_object_notify (G_OBJECT(check_menu_item), "active");
+     }
 }
 
 static void
 gtk_check_menu_item_toggle_size_request (GtkMenuItem *menu_item,
-					 guint16     *requisition)
+					 gint        *requisition)
 {
   g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (menu_item));
@@ -165,8 +197,10 @@ gtk_check_menu_item_set_show_toggle (GtkCheckMenuItem *menu_item,
 {
   g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (menu_item));
-  
+
+#if 0
   menu_item->always_show_toggle = always != FALSE;
+#endif  
 }
 
 void
@@ -175,25 +209,58 @@ gtk_check_menu_item_toggled (GtkCheckMenuItem *check_menu_item)
   gtk_signal_emit (GTK_OBJECT (check_menu_item), check_menu_item_signals[TOGGLED]);
 }
 
+/**
+ * gtk_check_menu_item_set_inconsistent:
+ * @check_menu_item: a #GtkCheckMenuItem
+ * @setting: %TRUE to display an "inconsistent" third state check
+ *
+ * If the user has selected a range of elements (such as some text or
+ * spreadsheet cells) that are affected by a boolean setting, and the
+ * current values in that range are inconsistent, you may want to
+ * display the check in an "in between" state. This function turns on
+ * "in between" display.  Normally you would turn off the inconsistent
+ * state again if the user explicitly selects a setting. This has to be
+ * done manually, gtk_check_menu_item_set_inconsistent() only affects
+ * visual appearance, it doesn't affect the semantics of the widget.
+ * 
+ **/
+void
+gtk_check_menu_item_set_inconsistent (GtkCheckMenuItem *check_menu_item,
+                                      gboolean          setting)
+{
+  g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (check_menu_item));
+  
+  setting = setting != FALSE;
+
+  if (setting != check_menu_item->inconsistent)
+    {
+      check_menu_item->inconsistent = setting;
+      gtk_widget_queue_draw (GTK_WIDGET (check_menu_item));
+      g_object_notify (G_OBJECT (check_menu_item), "inconsistent");
+    }
+}
+
+/**
+ * gtk_check_menu_item_get_inconsistent:
+ * @check_menu_item: a #GtkCheckMenuItem
+ * 
+ * Retrieves the value set by gtk_check_menu_item_set_inconsistent().
+ * 
+ * Return value: %TRUE if inconsistent
+ **/
+gboolean
+gtk_check_menu_item_get_inconsistent (GtkCheckMenuItem *check_menu_item)
+{
+  g_return_val_if_fail (GTK_IS_CHECK_MENU_ITEM (check_menu_item), FALSE);
+
+  return check_menu_item->inconsistent;
+}
+
 static void
 gtk_check_menu_item_init (GtkCheckMenuItem *check_menu_item)
 {
   check_menu_item->active = FALSE;
-  check_menu_item->always_show_toggle = FALSE;
-}
-
-static void
-gtk_check_menu_item_draw (GtkWidget    *widget,
-			  GdkRectangle *area)
-{
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (widget));
-  g_return_if_fail (area != NULL);
-
-  if (GTK_WIDGET_CLASS (parent_class)->draw)
-    (* GTK_WIDGET_CLASS (parent_class)->draw) (widget, area);
-
-  gtk_check_menu_item_draw_indicator (GTK_CHECK_MENU_ITEM (widget), area);
+  check_menu_item->always_show_toggle = TRUE;
 }
 
 static gint
@@ -281,7 +348,14 @@ gtk_real_check_menu_item_draw_indicator (GtkCheckMenuItem *check_menu_item,
 		  (state_type == GTK_STATE_PRELIGHT))
 		shadow_type = GTK_SHADOW_OUT;
 	    }
-	  
+
+          if (check_menu_item->inconsistent)
+            {
+              shadow_type = GTK_SHADOW_ETCHED_IN;
+              if (state_type == GTK_STATE_ACTIVE)
+                state_type = GTK_STATE_NORMAL;
+            }
+              
 	  gtk_paint_check (widget->style, widget->window,
 			   state_type, shadow_type,
 			   area, widget, "check",
@@ -289,3 +363,49 @@ gtk_real_check_menu_item_draw_indicator (GtkCheckMenuItem *check_menu_item,
 	}
     }
 }
+
+
+static void
+gtk_check_menu_item_get_property (GObject     *object,
+				  guint        prop_id,
+				  GValue      *value,
+				  GParamSpec  *pspec)
+{
+  GtkCheckMenuItem *checkitem = GTK_CHECK_MENU_ITEM (object);
+  
+  switch (prop_id)
+    {
+    case PROP_ACTIVE:
+      g_value_set_boolean (value, checkitem->active);
+      break;
+    case PROP_INCONSISTENT:
+      g_value_set_boolean (value, checkitem->inconsistent);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
+static void
+gtk_check_menu_item_set_property (GObject      *object,
+				  guint         prop_id,
+				  const GValue *value,
+				  GParamSpec   *pspec)
+{
+  GtkCheckMenuItem *checkitem = GTK_CHECK_MENU_ITEM (object);
+  
+  switch (prop_id)
+    {
+    case PROP_ACTIVE:
+      gtk_check_menu_item_set_active (checkitem, g_value_get_boolean (value));
+      break;
+    case PROP_INCONSISTENT:
+      gtk_check_menu_item_set_inconsistent (checkitem, g_value_get_boolean (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
