@@ -62,6 +62,7 @@ struct _GtkContainer
   guint need_resize : 1;
   guint resize_mode : 2;
   guint reallocate_redraws : 1;
+  guint has_focus_chain : 1;
   
   /* The list of children that requested a resize
    */
@@ -83,7 +84,7 @@ struct _GtkContainerClass
 				 gboolean	  include_internals,
 				 GtkCallback	  callback,
 				 gpointer	  callbabck_data);
-  gint (* focus)       		(GtkContainer	 *container,
+  gboolean (* focus)   		(GtkContainer	 *container,
 				 GtkDirectionType  direction);
   void (* set_focus_child)	(GtkContainer	 *container,
 				 GtkWidget	 *widget);
@@ -119,17 +120,24 @@ void    gtk_container_set_resize_mode    (GtkContainer     *container,
 
 void    gtk_container_check_resize       (GtkContainer     *container);
 
-void    gtk_container_foreach		 (GtkContainer	   *container,
-					  GtkCallback	    callback,
-					  gpointer	    callback_data);
-void    gtk_container_foreach_full	 (GtkContainer	   *container,
-					  GtkCallback	    callback,
-					  GtkCallbackMarshal marshal,
-					  gpointer	    callback_data,
-					  GtkDestroyNotify  notify);
-GList* gtk_container_children		 (GtkContainer	   *container);
-gint   gtk_container_focus		   (GtkContainer     *container,
-					    GtkDirectionType  direction);
+void     gtk_container_foreach      (GtkContainer       *container,
+				     GtkCallback         callback,
+				     gpointer            callback_data);
+void     gtk_container_foreach_full (GtkContainer       *container,
+				     GtkCallback         callback,
+				     GtkCallbackMarshal  marshal,
+				     gpointer            callback_data,
+				     GtkDestroyNotify    notify);
+GList*   gtk_container_children     (GtkContainer       *container);
+gboolean gtk_container_focus        (GtkContainer       *container,
+				     GtkDirectionType    direction);
+void     gtk_container_propagate_expose (GtkContainer   *container,
+					 GtkWidget      *child,
+					 GdkEventExpose *event);
+
+void     gtk_container_set_focus_chain  (GtkContainer   *container,
+                                         GList          *focusable_widgets);
+void     gtk_container_unset_focus_chain (GtkContainer  *container);
 
 /* Widget-level methods */
 
