@@ -552,6 +552,12 @@ gtk_entry_class_init (GtkEntryClass *class)
   add_move_binding (binding_set, GDK_Left, 0,
 		    GTK_MOVEMENT_VISUAL_POSITIONS, -1);
 
+  add_move_binding (binding_set, GDK_KP_Right, 0,
+		    GTK_MOVEMENT_VISUAL_POSITIONS, 1);
+  
+  add_move_binding (binding_set, GDK_KP_Left, 0,
+		    GTK_MOVEMENT_VISUAL_POSITIONS, -1);
+  
   add_move_binding (binding_set, GDK_f, GDK_CONTROL_MASK,
 		    GTK_MOVEMENT_LOGICAL_POSITIONS, 1);
   
@@ -563,6 +569,12 @@ gtk_entry_class_init (GtkEntryClass *class)
 
   add_move_binding (binding_set, GDK_Left, GDK_CONTROL_MASK,
 		    GTK_MOVEMENT_WORDS, -1);
+
+  add_move_binding (binding_set, GDK_KP_Right, GDK_CONTROL_MASK,
+		    GTK_MOVEMENT_WORDS, 1);
+
+  add_move_binding (binding_set, GDK_KP_Left, GDK_CONTROL_MASK,
+		    GTK_MOVEMENT_WORDS, -1);
   
   add_move_binding (binding_set, GDK_a, GDK_CONTROL_MASK,
 		    GTK_MOVEMENT_PARAGRAPH_ENDS, -1);
@@ -581,6 +593,12 @@ gtk_entry_class_init (GtkEntryClass *class)
 
   add_move_binding (binding_set, GDK_End, 0,
 		    GTK_MOVEMENT_DISPLAY_LINE_ENDS, 1);
+
+  add_move_binding (binding_set, GDK_KP_Home, 0,
+		    GTK_MOVEMENT_DISPLAY_LINE_ENDS, -1);
+
+  add_move_binding (binding_set, GDK_KP_End, 0,
+		    GTK_MOVEMENT_DISPLAY_LINE_ENDS, 1);
   
   add_move_binding (binding_set, GDK_Home, GDK_CONTROL_MASK,
 		    GTK_MOVEMENT_BUFFER_ENDS, -1);
@@ -588,12 +606,31 @@ gtk_entry_class_init (GtkEntryClass *class)
   add_move_binding (binding_set, GDK_End, GDK_CONTROL_MASK,
 		    GTK_MOVEMENT_BUFFER_ENDS, 1);
 
+  add_move_binding (binding_set, GDK_KP_Home, GDK_CONTROL_MASK,
+		    GTK_MOVEMENT_BUFFER_ENDS, -1);
+
+  add_move_binding (binding_set, GDK_KP_End, GDK_CONTROL_MASK,
+		    GTK_MOVEMENT_BUFFER_ENDS, 1);
+
+  /* Activate */
+
+  gtk_binding_entry_add_signal (binding_set, GDK_Return, 0,
+				"activate", 0);
+
+  gtk_binding_entry_add_signal (binding_set, GDK_KP_Enter, 0,
+				"activate", 0);
+  
   /* Deleting text */
   gtk_binding_entry_add_signal (binding_set, GDK_Delete, 0,
 				"delete_from_cursor", 2,
 				GTK_TYPE_ENUM, GTK_DELETE_CHARS,
 				GTK_TYPE_INT, 1);
 
+  gtk_binding_entry_add_signal (binding_set, GDK_KP_Delete, 0,
+				"delete_from_cursor", 2,
+				GTK_TYPE_ENUM, GTK_DELETE_CHARS,
+				GTK_TYPE_INT, 1);
+  
   gtk_binding_entry_add_signal (binding_set, GDK_d, GDK_CONTROL_MASK,
 				"delete_from_cursor", 2,
 				GTK_TYPE_ENUM, GTK_DELETE_CHARS,
@@ -609,6 +646,11 @@ gtk_entry_class_init (GtkEntryClass *class)
 				GTK_TYPE_ENUM, GTK_DELETE_WORD_ENDS,
 				GTK_TYPE_INT, 1);
 
+  gtk_binding_entry_add_signal (binding_set, GDK_KP_Delete, GDK_CONTROL_MASK,
+				"delete_from_cursor", 2,
+				GTK_TYPE_ENUM, GTK_DELETE_WORD_ENDS,
+				GTK_TYPE_INT, 1);
+  
   gtk_binding_entry_add_signal (binding_set, GDK_d, GDK_MOD1_MASK,
 				"delete_from_cursor", 2,
 				GTK_TYPE_ENUM, GTK_DELETE_WORD_ENDS,
@@ -712,7 +754,7 @@ gtk_entry_set_property (GObject         *object,
       break;
 
     case PROP_INVISIBLE_CHAR:
-      gtk_entry_set_invisible_char (entry, g_value_get_int (value));
+      gtk_entry_set_invisible_char (entry, g_value_get_uint (value));
       break;
 
     case PROP_ACTIVATES_DEFAULT:
@@ -754,7 +796,7 @@ gtk_entry_get_property (GObject         *object,
       g_value_set_boolean (value, entry->visible);
       break;
     case PROP_INVISIBLE_CHAR:
-      g_value_set_int (value, entry->invisible_char);
+      g_value_set_uint (value, entry->invisible_char);
       break;
     case PROP_ACTIVATES_DEFAULT:
       g_value_set_boolean (value, entry->activates_default);
@@ -846,7 +888,7 @@ gtk_entry_realize_cursor_gc (GtkEntry *entry)
       gdk_gc_set_rgb_fg_color (entry->cursor_gc, cursor_color);
     }
   else
-    entry->cursor_gc = gdk_gc_ref (GTK_WIDGET (entry)->style->bg_gc[GTK_STATE_SELECTED]);
+    entry->cursor_gc = gdk_gc_ref (GTK_WIDGET (entry)->style->base_gc[GTK_STATE_SELECTED]);
 }
 
 static void
@@ -882,7 +924,9 @@ gtk_entry_realize (GtkWidget *widget)
 			    GDK_BUTTON_RELEASE_MASK |
 			    GDK_BUTTON1_MOTION_MASK |
 			    GDK_BUTTON3_MOTION_MASK |
-			    GDK_POINTER_MOTION_HINT_MASK);
+			    GDK_POINTER_MOTION_HINT_MASK |
+                            GDK_ENTER_NOTIFY_MASK |
+			    GDK_LEAVE_NOTIFY_MASK);
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
 
   widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
@@ -1463,11 +1507,6 @@ gtk_entry_key_press (GtkWidget   *widget,
     /* Activate key bindings
      */
     return TRUE;
-  else if (event->keyval == GDK_Return)
-    {
-      gtk_widget_activate (widget);
-      return TRUE;
-    }
 
   return FALSE;
 }
@@ -1694,43 +1733,6 @@ gtk_entry_style_set	(GtkWidget      *widget,
     }
 }
 
-static char *
-strstr_len (const char *haystack,
-	    int         haystack_len,
-	    const char *needle)
-{
-  int i;
-
-  g_return_val_if_fail (haystack != NULL, NULL);
-  g_return_val_if_fail (needle != NULL, NULL);
-  
-  if (haystack_len < 0)
-    return strstr (haystack, needle);
-  else
-    {
-      const char *p = haystack;
-      int needle_len = strlen (needle);
-      const char *end = haystack + haystack_len - needle_len;
-
-      if (needle_len == 0)
-	return (char *)haystack;
-
-      while (*p && p <= end)
-	{
-	  for (i = 0; i < needle_len; i++)
-	    if (p[i] != needle[i])
-	      goto next;
-
-	  return (char *)p;
-
-	next:
-	  p += needle_len;
-	}
-    }
-
-  return NULL;
-}
-
 /* Default signal handlers
  */
 static void
@@ -1741,31 +1743,16 @@ gtk_entry_real_insert_text (GtkEntry    *entry,
 {
   gint index;
   gint n_chars;
-  gchar line_separator[7];
-  gint len;
-  gchar *p;
 
   if (new_text_length < 0)
     new_text_length = strlen (new_text);
 
-  /* We don't want to allow inserting paragraph delimeters
-   */
-  pango_find_paragraph_boundary (new_text, new_text_length, &new_text_length, NULL);
-
-  /* Or line separators - this is really painful
-   */
-  len = g_unichar_to_utf8 (0x2028, line_separator); /* 0x2028 == LS */
-  line_separator[len] = '\0';
-  
-  p = strstr_len (new_text, new_text_length, line_separator);
-  if (p)
-    new_text_length = p - new_text;
-
   n_chars = g_utf8_strlen (new_text, new_text_length);
   if (entry->text_max_length > 0 && n_chars + entry->text_length > entry->text_max_length)
     {
       gdk_beep ();
       n_chars = entry->text_max_length - entry->text_length;
+      new_text_length = g_utf8_offset_to_pointer (new_text, n_chars) - new_text;
     }
 
   if (new_text_length + entry->n_bytes + 1 > entry->text_size)
@@ -2167,6 +2154,8 @@ gtk_entry_create_layout (GtkEntry *entry,
   gint preedit_length = 0;
   PangoAttrList *preedit_attrs = NULL;
 
+  pango_layout_set_single_paragraph_mode (layout, TRUE);
+  
   if (include_preedit)
     {
       gtk_im_context_get_preedit_string (entry->im_context,
@@ -2372,17 +2361,17 @@ gtk_entry_draw_text (GtkEntry *entry)
 	      rect.width = (ranges[2*i + 1] - ranges[2*i]) / PANGO_SCALE;
 	      rect.height = logical_rect.height / PANGO_SCALE;
 	      
-	      gdk_draw_rectangle (entry->text_area, widget->style->bg_gc [GTK_STATE_SELECTED], TRUE,
+	      gdk_draw_rectangle (entry->text_area, widget->style->base_gc [GTK_STATE_SELECTED], TRUE,
 				  rect.x, rect.y, rect.width, rect.height);
 
 	      gdk_region_union_with_rect (clip_region, &rect);
 	    }
 
-	  gdk_gc_set_clip_region (widget->style->fg_gc [GTK_STATE_SELECTED], clip_region);
-	  gdk_draw_layout (entry->text_area, widget->style->fg_gc [GTK_STATE_SELECTED], 
+	  gdk_gc_set_clip_region (widget->style->text_gc [GTK_STATE_SELECTED], clip_region);
+	  gdk_draw_layout (entry->text_area, widget->style->text_gc [GTK_STATE_SELECTED], 
 			   x, y,
 			   layout);
-	  gdk_gc_set_clip_region (widget->style->fg_gc [GTK_STATE_SELECTED], NULL);
+	  gdk_gc_set_clip_region (widget->style->text_gc [GTK_STATE_SELECTED], NULL);
 	  
 	  gdk_region_destroy (clip_region);
 	  g_free (ranges);
@@ -2416,7 +2405,7 @@ gtk_entry_draw_cursor (GtkEntry  *entry,
 		     xoffset + strong_x, text_area_height - INNER_BORDER);
       
       if (weak_x != strong_x)
-	gdk_draw_line (entry->text_area, widget->style->fg_gc[GTK_STATE_NORMAL], 
+	gdk_draw_line (entry->text_area, widget->style->text_gc[GTK_STATE_NORMAL], 
 		       xoffset + weak_x, INNER_BORDER,
 		       xoffset + weak_x, text_area_height - INNER_BORDER);
     }
