@@ -37,12 +37,12 @@ static void gtk_event_box_size_allocate            (GtkWidget        *widget,
 						    GtkAllocation    *allocation);
 static void gtk_event_box_paint                    (GtkWidget         *widget,
 						    GdkRectangle      *area);
-static void gtk_event_box_draw                     (GtkWidget         *widget,
-						   GdkRectangle       *area);
 static gint gtk_event_box_expose                   (GtkWidget         *widget,
 						   GdkEventExpose     *event);
 
 
+static GtkBinClass *parent_class = NULL;
+
 GtkType
 gtk_event_box_get_type (void)
 {
@@ -73,12 +73,13 @@ gtk_event_box_class_init (GtkEventBoxClass *class)
 {
   GtkWidgetClass *widget_class;
 
+  parent_class = g_type_class_peek_parent (class);
+  
   widget_class = (GtkWidgetClass*) class;
 
   widget_class->realize = gtk_event_box_realize;
   widget_class->size_request = gtk_event_box_size_request;
   widget_class->size_allocate = gtk_event_box_size_allocate;
-  widget_class->draw = gtk_event_box_draw;
   widget_class->expose_event = gtk_event_box_expose;
 }
 
@@ -203,56 +204,19 @@ gtk_event_box_paint (GtkWidget    *widget,
 		      0, 0, -1, -1);
 }
 
-static void
-gtk_event_box_draw (GtkWidget    *widget,
-		    GdkRectangle *area)
-{
-  GtkBin *bin;
-  GdkRectangle tmp_area;
-  GdkRectangle child_area;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_EVENT_BOX (widget));
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      bin = GTK_BIN (widget);
-      tmp_area = *area;
-      tmp_area.x -= GTK_CONTAINER (widget)->border_width;
-      tmp_area.y -= GTK_CONTAINER (widget)->border_width;
-
-      gtk_event_box_paint (widget, &tmp_area);
-      
-      if (bin->child)
-	{
-	  if (gtk_widget_intersect (bin->child, &tmp_area, &child_area))
-	    gtk_widget_draw (bin->child, &child_area);
-	}
-    }
-}
-
 static gint
 gtk_event_box_expose (GtkWidget      *widget,
 		     GdkEventExpose *event)
 {
-  GtkBin *bin;
-  GdkEventExpose child_event;
-
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_EVENT_BOX (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
   if (GTK_WIDGET_DRAWABLE (widget))
     {
-      bin = GTK_BIN (widget);
-
       gtk_event_box_paint (widget, &event->area);
       
-      child_event = *event;
-      if (bin->child &&
-	  GTK_WIDGET_NO_WINDOW (bin->child) &&
-	  gtk_widget_intersect (bin->child, &event->area, &child_event.area))
-	gtk_widget_event (bin->child, (GdkEvent*) &child_event);
+      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
     }
 
   return FALSE;
