@@ -331,7 +331,9 @@ gtk_hsv_realize (GtkWidget *widget)
   attr.event_mask |= (GDK_KEY_PRESS_MASK
                       | GDK_BUTTON_PRESS_MASK
 		      | GDK_BUTTON_RELEASE_MASK
-		      | GDK_POINTER_MOTION_MASK);
+		      | GDK_POINTER_MOTION_MASK
+                      | GDK_ENTER_NOTIFY_MASK
+                      | GDK_LEAVE_NOTIFY_MASK);
   
   attr_mask = GDK_WA_X | GDK_WA_Y;
   
@@ -686,8 +688,12 @@ compute_sv (GtkHSV  *hsv,
 	{
 	  if (*v > 1.0)
 	    *v = 1.0;
-	  
-	  *s = (y - sy - *v * (vy - sy)) / (*v * (hy - vy));
+
+	  if (fabs (hy - vy) < fabs (hx - vx))
+	    *s = (x - sx - *v * (vx - sx)) / (*v * (hx - vx));
+	  else
+	    *s = (y - sy - *v * (vy - sy)) / (*v * (hy - vy));
+	    
 	  if (*s < 0.0)
 	    *s = 0.0;
 	  else if (*s > 1.0)
@@ -949,7 +955,7 @@ paint_ring (GtkHSV      *hsv,
 	  dx = xx + x - center;
 	  
 	  dist = dx * dx + dy * dy;
-	  if (dist < (inner * inner) || dist > (outer * outer))
+	  if (dist < ((inner-1) * (inner-1)) || dist > ((outer+1) * (outer+1)))
 	    {
 	      *p++ = 0;
 	      *p++ = 0;
@@ -1271,14 +1277,17 @@ paint_triangle (GtkHSV      *hsv,
     gdk_rgb_gc_set_foreground (priv->gc, 0x000000);
   else
     gdk_rgb_gc_set_foreground (priv->gc, 0xffffff);
+
+#define OUTER_RADIUS 4
+#define INNER_RADIUS 3 
   
   gdk_draw_arc (drawable, priv->gc, FALSE,
-		xx - 3, yy - 3,
-		6, 6,
+		xx - OUTER_RADIUS, yy - OUTER_RADIUS,
+		OUTER_RADIUS * 2, OUTER_RADIUS * 2,
 		0, 360 * 64);
   gdk_draw_arc (drawable, priv->gc, FALSE,
-		xx - 2, yy - 2,
-		4, 4,
+		xx - INNER_RADIUS, yy - INNER_RADIUS,
+		INNER_RADIUS * 2, INNER_RADIUS * 2,
 		0, 360 * 64);
 }
 
