@@ -29,6 +29,7 @@
 
 #include <gdk/gdk.h>
 #include <gtk/gtkmisc.h>
+#include <gtk/gtkwindow.h>
 
 
 #ifdef __cplusplus
@@ -47,21 +48,30 @@ extern "C" {
 typedef struct _GtkLabel       GtkLabel;
 typedef struct _GtkLabelClass  GtkLabelClass;
 
-typedef struct _GtkLabelWord   GtkLabelWord;
+typedef struct _GtkLabelSelectionInfo GtkLabelSelectionInfo;
 
 struct _GtkLabel
 {
   GtkMisc misc;
 
-  gchar    *label;
-  gchar    *pattern;
-
+  /*< private >*/
+  gchar  *label;
   guint   jtype : 2;
   guint   wrap : 1;
+  guint   use_underline : 1;
+  guint   use_markup : 1;
+
+  guint   mnemonic_keyval;
   
-  /*< private >*/
-  PangoLayout *layout;
+  gchar  *text; 
   PangoAttrList *attrs;
+  
+  PangoLayout *layout;
+
+  GtkWidget *mnemonic_widget;
+  GtkWindow *mnemonic_window;
+  
+  GtkLabelSelectionInfo *select_info;
 };
 
 struct _GtkLabelClass
@@ -69,41 +79,57 @@ struct _GtkLabelClass
   GtkMiscClass parent_class;
 };
 
-GtkType    gtk_label_get_type       (void) G_GNUC_CONST;
-GtkWidget *gtk_label_new            (const char       *str);
-void       gtk_label_set_text       (GtkLabel         *label,
-				     const char       *str);
-void       gtk_label_set_attributes (GtkLabel         *label,
-                                     PangoAttrList    *attrs);
-
-void  gtk_label_set_markup            (GtkLabel    *label,
-                                       const gchar *str);
-guint gtk_label_set_markup_with_accel (GtkLabel    *label,
-                                       const gchar *str);
-                                            
-
-/* Temporarily commented out until memory management behavior is figured out *//* gchar *    gtk_label_get_text      (GtkLabel         *label); */
-void       gtk_label_set_justify   (GtkLabel         *label,
-				    GtkJustification  jtype);
-void       gtk_label_set_pattern   (GtkLabel         *label,
-				    const gchar      *pattern);
-void       gtk_label_set_line_wrap (GtkLabel         *label,
-				    gboolean          wrap);
-/* Convenience function to set the name and pattern by parsing
- * a string with embedded underscores, and return the appropriate
- * key symbol for the accelerator.
- */
-guint       gtk_label_parse_uline   (GtkLabel         *label,
-				     const gchar      *string);
+GtkType               gtk_label_get_type          (void) G_GNUC_CONST;
+GtkWidget*            gtk_label_new               (const char       *str);
+GtkWidget*            gtk_label_new_with_mnemonic (const char       *str);
+void                  gtk_label_set_text          (GtkLabel         *label,
+						   const char       *str);
+G_CONST_RETURN gchar* gtk_label_get_text          (GtkLabel         *label);
+void                  gtk_label_set_attributes    (GtkLabel         *label,
+						   PangoAttrList    *attrs);
+void                  gtk_label_set_markup        (GtkLabel         *label,
+						   const gchar      *str);
+void     gtk_label_set_markup_with_mnemonic       (GtkLabel         *label,
+						   const gchar      *str);
+guint    gtk_label_get_mnemonic_keyval            (GtkLabel         *label);
+void     gtk_label_set_mnemonic_widget            (GtkLabel         *label,
+						   GtkWidget        *widget);
+void     gtk_label_set_text_with_mnemonic         (GtkLabel         *label,
+						   const gchar      *str);
+void     gtk_label_set_justify                    (GtkLabel         *label,
+						   GtkJustification  jtype);
+void     gtk_label_set_pattern                    (GtkLabel         *label,
+						   const gchar      *pattern);
+void     gtk_label_set_line_wrap                  (GtkLabel         *label,
+						   gboolean          wrap);
+void     gtk_label_set_selectable                 (GtkLabel         *label,
+						   gboolean          setting);
+gboolean gtk_label_get_selectable                 (GtkLabel         *label);
+void     gtk_label_select_region                  (GtkLabel         *label,
+						   gint              start_offset,
+						   gint              end_offset);
+void     gtk_label_get_layout_offsets             (GtkLabel         *label,
+						   gint             *x,
+						   gint             *y);
 
 #ifndef	GTK_DISABLE_COMPAT_H
 #  define gtk_label_set				gtk_label_set_text
 #endif	/* GTK_DISABLE_COMPAT_H */
 
+#ifndef GTK_DISABLE_DEPRECATED
 /* Deprecated */
 void       gtk_label_get           (GtkLabel          *label,
                                     char             **str);
 
+/* Convenience function to set the name and pattern by parsing
+ * a string with embedded underscores, and return the appropriate
+ * key symbol for the accelerator.
+ */
+guint gtk_label_parse_uline            (GtkLabel    *label,
+					const gchar *string);
+
+#endif /* GTK_DISABLE_DEPRECATED */
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
