@@ -26,11 +26,7 @@
 
 #include "gdkconfig.h"
 
-#ifdef GDK_WINDOWING_X11
-#include <X11/Xlocale.h>	/* so we get the right setlocale */
-#else
 #include <locale.h>
-#endif
 
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
 #include <libintl.h>
@@ -948,6 +944,9 @@ gtk_main_do_event (GdkEvent *event)
     case GDK_DROP_START:
       gtk_drag_dest_handle_event (event_widget, event);
       break;
+    default:
+      g_assert_not_reached ();
+      break;
     }
   
   tmp_list = current_events;
