@@ -802,7 +802,44 @@ gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
 
 /* VOID:ENUM (gtkmarshal.list:50) */
 
-/* VOID:ENUM,FLOAT (gtkmarshal.list:51) */
+/* VOID:ENUM,ENUM (gtkmarshal.list:51) */
+void
+gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
+                             GValue       *return_value,
+                             guint         n_param_values,
+                             const GValue *param_values,
+                             gpointer      invocation_hint,
+                             gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__ENUM_ENUM) (gpointer     data1,
+                                                gint         arg_1,
+                                                gint         arg_2,
+                                                gpointer     data2);
+  register GMarshalFunc_VOID__ENUM_ENUM callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__ENUM_ENUM) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_value_get_enum (param_values + 1),
+            g_value_get_enum (param_values + 2),
+            data2);
+}
+
+/* VOID:ENUM,FLOAT (gtkmarshal.list:52) */
 void
 gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                               GValue       *return_value,
@@ -839,7 +876,7 @@ gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM,FLOAT,BOOL (gtkmarshal.list:52) */
+/* VOID:ENUM,FLOAT,BOOL (gtkmarshal.list:53) */
 void
 gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                       GValue       *return_value,
@@ -878,7 +915,7 @@ gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM,INT (gtkmarshal.list:53) */
+/* VOID:ENUM,INT (gtkmarshal.list:54) */
 void
 gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                             GValue       *return_value,
@@ -915,7 +952,7 @@ gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM,INT,BOOLEAN (gtkmarshal.list:54) */
+/* VOID:ENUM,INT,BOOLEAN (gtkmarshal.list:55) */
 void
 gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                     GValue       *return_value,
@@ -954,11 +991,11 @@ gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT (gtkmarshal.list:55) */
+/* VOID:INT (gtkmarshal.list:56) */
 
-/* VOID:INT,INT (gtkmarshal.list:56) */
+/* VOID:INT,INT (gtkmarshal.list:57) */
 
-/* VOID:INT,INT,BOXED (gtkmarshal.list:57) */
+/* VOID:INT,INT,BOXED (gtkmarshal.list:58) */
 void
 gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                  GValue       *return_value,
@@ -997,7 +1034,7 @@ gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT,INT (gtkmarshal.list:58) */
+/* VOID:INT,INT,INT (gtkmarshal.list:59) */
 void
 gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                GValue       *return_value,
@@ -1036,9 +1073,9 @@ gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT (gtkmarshal.list:59) */
+/* VOID:OBJECT (gtkmarshal.list:60) */
 
-/* VOID:OBJECT,BOOLEAN (gtkmarshal.list:60) */
+/* VOID:OBJECT,BOOLEAN (gtkmarshal.list:61) */
 void
 gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                   GValue       *return_value,
@@ -1075,7 +1112,7 @@ gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,BOXED,BOXED (gtkmarshal.list:61) */
+/* VOID:OBJECT,BOXED,BOXED (gtkmarshal.list:62) */
 void
 gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                       GValue       *return_value,
@@ -1114,7 +1151,7 @@ gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshal.list:62) */
+/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshal.list:63) */
 void
 gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                           GValue       *return_value,
@@ -1155,7 +1192,7 @@ gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT (gtkmarshal.list:63) */
+/* VOID:OBJECT,INT,INT (gtkmarshal.list:64) */
 void
 gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                   GValue       *return_value,
@@ -1194,7 +1231,7 @@ gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshal.list:64) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshal.list:65) */
 void
 gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                   GValue       *return_value,
@@ -1239,7 +1276,7 @@ gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,OBJECT (gtkmarshal.list:65) */
+/* VOID:OBJECT,OBJECT (gtkmarshal.list:66) */
 void
 gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                  GValue       *return_value,
@@ -1276,7 +1313,7 @@ gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,STRING,STRING (gtkmarshal.list:66) */
+/* VOID:OBJECT,STRING,STRING (gtkmarshal.list:67) */
 void
 gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                         GValue       *return_value,
@@ -1315,7 +1352,7 @@ gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,UINT (gtkmarshal.list:67) */
+/* VOID:OBJECT,UINT (gtkmarshal.list:68) */
 void
 gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                GValue       *return_value,
@@ -1352,9 +1389,9 @@ gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER (gtkmarshal.list:68) */
+/* VOID:POINTER (gtkmarshal.list:69) */
 
-/* VOID:POINTER,INT (gtkmarshal.list:69) */
+/* VOID:POINTER,INT (gtkmarshal.list:70) */
 void
 gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                GValue       *return_value,
@@ -1391,7 +1428,7 @@ gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER,POINTER (gtkmarshal.list:70) */
+/* VOID:POINTER,POINTER,POINTER (gtkmarshal.list:71) */
 void
 gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                            GValue       *return_value,
@@ -1430,7 +1467,7 @@ gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,UINT (gtkmarshal.list:71) */
+/* VOID:POINTER,UINT (gtkmarshal.list:72) */
 void
 gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                 GValue       *return_value,
@@ -1467,11 +1504,11 @@ gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING (gtkmarshal.list:72) */
+/* VOID:STRING (gtkmarshal.list:73) */
 
-/* VOID:STRING,INT,POINTER (gtkmarshal.list:73) */
+/* VOID:STRING,INT,POINTER (gtkmarshal.list:74) */
 
-/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshal.list:74) */
+/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshal.list:75) */
 void
 gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                GValue       *return_value,
@@ -1514,7 +1551,7 @@ gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT,STRING (gtkmarshal.list:75) */
+/* VOID:UINT,STRING (gtkmarshal.list:76) */
 void
 gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                GValue       *return_value,
@@ -1551,5 +1588,5 @@ gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:VOID (gtkmarshal.list:76) */
+/* VOID:VOID (gtkmarshal.list:77) */
 
