@@ -191,7 +191,15 @@ extern void gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
 /* VOID:ENUM (gtkmarshal.list:50) */
 #define gtk_marshal_VOID__ENUM	g_cclosure_marshal_VOID__ENUM
 
-/* VOID:ENUM,FLOAT (gtkmarshal.list:51) */
+/* VOID:ENUM,ENUM (gtkmarshal.list:51) */
+extern void gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
+                                         GValue       *return_value,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint,
+                                         gpointer      marshal_data);
+
+/* VOID:ENUM,FLOAT (gtkmarshal.list:52) */
 extern void gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -199,7 +207,7 @@ extern void gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:ENUM,FLOAT,BOOL (gtkmarshal.list:52) */
+/* VOID:ENUM,FLOAT,BOOL (gtkmarshal.list:53) */
 extern void gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -208,7 +216,7 @@ extern void gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                   gpointer      marshal_data);
 #define gtk_marshal_VOID__ENUM_FLOAT_BOOL	gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN
 
-/* VOID:ENUM,INT (gtkmarshal.list:53) */
+/* VOID:ENUM,INT (gtkmarshal.list:54) */
 extern void gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -216,7 +224,7 @@ extern void gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
 
-/* VOID:ENUM,INT,BOOLEAN (gtkmarshal.list:54) */
+/* VOID:ENUM,INT,BOOLEAN (gtkmarshal.list:55) */
 extern void gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -224,11 +232,11 @@ extern void gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:INT (gtkmarshal.list:55) */
+/* VOID:INT (gtkmarshal.list:56) */
 
-/* VOID:INT,INT (gtkmarshal.list:56) */
+/* VOID:INT,INT (gtkmarshal.list:57) */
 
-/* VOID:INT,INT,BOXED (gtkmarshal.list:57) */
+/* VOID:INT,INT,BOXED (gtkmarshal.list:58) */
 extern void gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -236,7 +244,7 @@ extern void gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:INT,INT,INT (gtkmarshal.list:58) */
+/* VOID:INT,INT,INT (gtkmarshal.list:59) */
 extern void gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -244,10 +252,10 @@ extern void gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:OBJECT (gtkmarshal.list:59) */
+/* VOID:OBJECT (gtkmarshal.list:60) */
 #define gtk_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
 
-/* VOID:OBJECT,BOOLEAN (gtkmarshal.list:60) */
+/* VOID:OBJECT,BOOLEAN (gtkmarshal.list:61) */
 extern void gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -255,7 +263,7 @@ extern void gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,BOXED (gtkmarshal.list:61) */
+/* VOID:OBJECT,BOXED,BOXED (gtkmarshal.list:62) */
 extern void gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -263,7 +271,7 @@ extern void gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                   gpointer      invocation_hint,
                                                   gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshal.list:62) */
+/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshal.list:63) */
 extern void gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -271,7 +279,7 @@ extern void gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                       gpointer      invocation_hint,
                                                       gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT (gtkmarshal.list:63) */
+/* VOID:OBJECT,INT,INT (gtkmarshal.list:64) */
 extern void gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -279,7 +287,7 @@ extern void gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshal.list:64) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshal.list:65) */
 extern void gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                               GValue       *return_value,
                                                               guint         n_param_values,
@@ -287,7 +295,7 @@ extern void gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *clos
                                                               gpointer      invocation_hint,
                                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,OBJECT (gtkmarshal.list:65) */
+/* VOID:OBJECT,OBJECT (gtkmarshal.list:66) */
 extern void gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -295,7 +303,7 @@ extern void gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:OBJECT,STRING,STRING (gtkmarshal.list:66) */
+/* VOID:OBJECT,STRING,STRING (gtkmarshal.list:67) */
 extern void gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -303,7 +311,7 @@ extern void gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT (gtkmarshal.list:67) */
+/* VOID:OBJECT,UINT (gtkmarshal.list:68) */
 extern void gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -311,9 +319,9 @@ extern void gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:POINTER (gtkmarshal.list:68) */
+/* VOID:POINTER (gtkmarshal.list:69) */
 
-/* VOID:POINTER,INT (gtkmarshal.list:69) */
+/* VOID:POINTER,INT (gtkmarshal.list:70) */
 extern void gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -321,7 +329,7 @@ extern void gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER (gtkmarshal.list:70) */
+/* VOID:POINTER,POINTER,POINTER (gtkmarshal.list:71) */
 extern void gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                        GValue       *return_value,
                                                        guint         n_param_values,
@@ -329,7 +337,7 @@ extern void gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                        gpointer      invocation_hint,
                                                        gpointer      marshal_data);
 
-/* VOID:POINTER,UINT (gtkmarshal.list:71) */
+/* VOID:POINTER,UINT (gtkmarshal.list:72) */
 extern void gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -337,12 +345,12 @@ extern void gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING (gtkmarshal.list:72) */
+/* VOID:STRING (gtkmarshal.list:73) */
 #define gtk_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
 
-/* VOID:STRING,INT,POINTER (gtkmarshal.list:73) */
+/* VOID:STRING,INT,POINTER (gtkmarshal.list:74) */
 
-/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshal.list:74) */
+/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshal.list:75) */
 extern void gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                            GValue       *return_value,
                                                            guint         n_param_values,
@@ -350,7 +358,7 @@ extern void gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure
                                                            gpointer      invocation_hint,
                                                            gpointer      marshal_data);
 
-/* VOID:UINT,STRING (gtkmarshal.list:75) */
+/* VOID:UINT,STRING (gtkmarshal.list:76) */
 extern void gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -358,7 +366,7 @@ extern void gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:VOID (gtkmarshal.list:76) */
+/* VOID:VOID (gtkmarshal.list:77) */
 
 #ifdef __cplusplus
 }
