@@ -24,6 +24,16 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+/*
+ * NOTE this widget is considered too specialized/little-used for
+ * GTK+, and will in the future be moved to some other package.  If
+ * your application needs this widget, feel free to use it, as the
+ * widget does work and is useful in some applications; it's just not
+ * of general interest. However, we are not accepting new features for
+ * the widget, and it will eventually move out of the GTK+
+ * distribution.
+ */
+
 #ifndef __GTK_RULER_H__
 #define __GTK_RULER_H__
 
@@ -63,13 +73,13 @@ struct _GtkRuler
   gint slider_size;
 
   /* The upper limit of the ruler (in points) */
-  gfloat lower;
+  gdouble lower;
   /* The lower limit of the ruler */
-  gfloat upper;
+  gdouble upper;
   /* The position of the mark on the ruler */
-  gfloat position;
+  gdouble position;
   /* The maximum size of the ruler */
-  gfloat max_size;
+  gdouble max_size;
 };
 
 struct _GtkRulerClass
@@ -86,8 +96,8 @@ struct _GtkRulerMetric
   gchar *abbrev;
   /* This should be points_per_unit. This is the size of the unit
    * in 1/72nd's of an inch and has nothing to do with screen pixels */
-  gfloat pixels_per_unit;
-  gfloat ruler_scale[10];
+  gdouble pixels_per_unit;
+  gdouble ruler_scale[10];
   gint subdivide[5];        /* five possible modes of subdivision */
 };
 
@@ -96,10 +106,10 @@ GtkType gtk_ruler_get_type   (void) G_GNUC_CONST;
 void    gtk_ruler_set_metric (GtkRuler      *ruler,
 			      GtkMetricType  metric);
 void    gtk_ruler_set_range  (GtkRuler      *ruler,
-			      gfloat         lower,
-			      gfloat         upper,
-			      gfloat         position,
-			      gfloat         max_size);
+			      gdouble        lower,
+			      gdouble        upper,
+			      gdouble        position,
+			      gdouble        max_size);
 void    gtk_ruler_draw_ticks (GtkRuler      *ruler);
 void    gtk_ruler_draw_pos   (GtkRuler      *ruler);
 
