@@ -90,8 +90,6 @@ static void gtk_scrolled_window_destroy            (GtkObject              *obje
 static void gtk_scrolled_window_finalize           (GObject                *object);
 static void gtk_scrolled_window_map                (GtkWidget              *widget);
 static void gtk_scrolled_window_unmap              (GtkWidget              *widget);
-static void gtk_scrolled_window_draw               (GtkWidget              *widget,
-						    GdkRectangle           *area);
 static gint gtk_scrolled_window_expose             (GtkWidget              *widget,
 						    GdkEventExpose         *event);
 static void gtk_scrolled_window_size_request       (GtkWidget              *widget,
@@ -162,7 +160,6 @@ gtk_scrolled_window_class_init (GtkScrolledWindowClass *class)
 
   widget_class->map = gtk_scrolled_window_map;
   widget_class->unmap = gtk_scrolled_window_unmap;
-  widget_class->draw = gtk_scrolled_window_draw;
   widget_class->expose_event = gtk_scrolled_window_expose;
   widget_class->size_request = gtk_scrolled_window_size_request;
   widget_class->size_allocate = gtk_scrolled_window_size_allocate;
@@ -473,6 +470,15 @@ gtk_scrolled_window_set_placement (GtkScrolledWindow *scrolled_window,
     }
 }
 
+/**
+ * gtk_scrolled_window_set_shadow_type:
+ * @scrolled_window: a #GtkScrolledWindow
+ * @type: kind of shadow to draw around scrolled window contents
+ *
+ * Changes the type of shadow drawn around the contents of
+ * @scrolled_window.
+ * 
+ **/
 void
 gtk_scrolled_window_set_shadow_type (GtkScrolledWindow *scrolled_window,
 				     GtkShadowType      type)
@@ -588,58 +594,15 @@ gtk_scrolled_window_paint (GtkWidget    *widget,
     }
 }
 
-static void
-gtk_scrolled_window_draw (GtkWidget    *widget,
-			  GdkRectangle *area)
-{
-  GtkScrolledWindow *scrolled_window;
-  GtkBin *bin;
-  GdkRectangle child_area;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_SCROLLED_WINDOW (widget));
-  g_return_if_fail (area != NULL);
-  
-  scrolled_window = GTK_SCROLLED_WINDOW (widget);
-  bin = GTK_BIN (widget);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      gtk_scrolled_window_paint (widget, area);
-  
-      if (bin->child && GTK_WIDGET_VISIBLE (bin->child) &&
-	  gtk_widget_intersect (bin->child, area, &child_area))
-	gtk_widget_draw (bin->child, &child_area);
-      
-      if (GTK_WIDGET_VISIBLE (scrolled_window->hscrollbar) &&
-	  gtk_widget_intersect (scrolled_window->hscrollbar, area, &child_area))
-	gtk_widget_draw (scrolled_window->hscrollbar, &child_area);
-      
-      if (GTK_WIDGET_VISIBLE (scrolled_window->vscrollbar) &&
-	  gtk_widget_intersect (scrolled_window->vscrollbar, area, &child_area))
-	gtk_widget_draw (scrolled_window->vscrollbar, &child_area);
-    }
-}
-  
 static gint
 gtk_scrolled_window_expose (GtkWidget      *widget,
 			    GdkEventExpose *event)
 {
-  GtkBin *bin = GTK_BIN (widget);
-  GdkEventExpose child_event;
-
   if (GTK_WIDGET_DRAWABLE (widget))
     {
       gtk_scrolled_window_paint (widget, &event->area);
 
-      if (bin->child && GTK_WIDGET_VISIBLE (bin->child) && GTK_WIDGET_NO_WINDOW (bin->child))
-	{
-	  child_event = *event;	  
-	  if (gtk_widget_intersect (bin->child, &event->area, &child_event.area))
-	    gtk_widget_event (bin->child, (GdkEvent*) &child_event);
-	}
-
-      /* We rely on our knowledge that scrollbars are !NO_WINDOW widgets */
+      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
     }
 
   return FALSE;
@@ -703,20 +666,14 @@ gtk_scrolled_window_size_request (GtkWidget      *widget,
   
   if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
     {
-      static guint quark_aux_info = 0;
-
-      if (!quark_aux_info)
-	quark_aux_info = g_quark_from_static_string ("gtk-aux-info");
-
       gtk_widget_size_request (bin->child, &child_requisition);
 
       if (scrolled_window->hscrollbar_policy == GTK_POLICY_NEVER)
 	requisition->width += child_requisition.width;
       else
 	{
-	  GtkWidgetAuxInfo *aux_info;
+	  GtkWidgetAuxInfo *aux_info = _gtk_widget_get_aux_info (bin->child, FALSE);
 
-	  aux_info = gtk_object_get_data_by_id (GTK_OBJECT (bin->child), quark_aux_info);
 	  if (aux_info && aux_info->width > 0)
 	    {
 	      requisition->width += aux_info->width;
@@ -730,9 +687,7 @@ gtk_scrolled_window_size_request (GtkWidget      *widget,
 	requisition->height += child_requisition.height;
       else
 	{
-	  GtkWidgetAuxInfo *aux_info;
-
-	  aux_info = gtk_object_get_data_by_id (GTK_OBJECT (bin->child), quark_aux_info);
+	  GtkWidgetAuxInfo *aux_info = _gtk_widget_get_aux_info (bin->child, FALSE);
 	  if (aux_info && aux_info->height > 0)
 	    {
 	      requisition->height += aux_info->height;
@@ -986,7 +941,7 @@ gtk_scrolled_window_scroll_event (GtkWidget *widget,
   if (range && GTK_WIDGET_VISIBLE (range))
     {
       GtkAdjustment *adj = GTK_RANGE (range)->adjustment;
-      gfloat new_value;
+      gdouble new_value;
 
       if (event->direction == GDK_SCROLL_UP || event->direction == GDK_SCROLL_LEFT)
 	new_value = adj->value - adj->page_increment / 2;
