@@ -79,13 +79,12 @@ struct _GtkSpinButton
   GtkAdjustment *adjustment;
   
   GdkWindow *panel;
-  GtkShadowType shadow_type;
   
   guint32 timer;
   guint32 ev_time;
   
-  gfloat climb_rate;
-  gfloat timer_step;
+  gdouble climb_rate;
+  gdouble timer_step;
   
   GtkSpinButtonUpdatePolicy update_policy;
   
@@ -105,8 +104,9 @@ struct _GtkSpinButtonClass
   GtkEntryClass parent_class;
 
   gint (*input)  (GtkSpinButton *spin_button,
-		  gfloat        *new_value);
+		  gdouble       *new_value);
   gint (*output) (GtkSpinButton *spin_button);
+  void (*value_changed) (GtkSpinButton *spin_button);
 };
 
 
@@ -114,13 +114,17 @@ GtkType		gtk_spin_button_get_type	   (void) G_GNUC_CONST;
 
 void		gtk_spin_button_configure	   (GtkSpinButton  *spin_button,
 						    GtkAdjustment  *adjustment,
-						    gfloat	    climb_rate,
+						    gdouble	    climb_rate,
 						    guint	    digits);
 
 GtkWidget*	gtk_spin_button_new		   (GtkAdjustment  *adjustment,
-						    gfloat	    climb_rate,
+						    gdouble	    climb_rate,
 						    guint	    digits);
 
+GtkWidget*	gtk_spin_button_new_with_range	   (gdouble  min,
+						    gdouble  max,
+						    gdouble  step);
+
 void		gtk_spin_button_set_adjustment	   (GtkSpinButton  *spin_button,
 						    GtkAdjustment  *adjustment);
 
@@ -129,12 +133,20 @@ GtkAdjustment*	gtk_spin_button_get_adjustment	   (GtkSpinButton  *spin_button);
 void		gtk_spin_button_set_digits	   (GtkSpinButton  *spin_button,
 						    guint	    digits);
 
-gfloat		gtk_spin_button_get_value_as_float (GtkSpinButton  *spin_button);
+void		gtk_spin_button_set_increments	   (GtkSpinButton  *spin_button,
+						    gdouble         step,
+						    gdouble         page);
+
+void		gtk_spin_button_set_range	   (GtkSpinButton  *spin_button,
+						    gdouble         min,
+						    gdouble         max);
+
+gdouble		gtk_spin_button_get_value_as_float (GtkSpinButton  *spin_button);
 
 gint		gtk_spin_button_get_value_as_int   (GtkSpinButton  *spin_button);
 
 void		gtk_spin_button_set_value	   (GtkSpinButton  *spin_button, 
-						    gfloat	    value);
+						    gdouble	    value);
 
 void		gtk_spin_button_set_update_policy  (GtkSpinButton  *spin_button,
 						    GtkSpinButtonUpdatePolicy  policy);
@@ -144,14 +156,11 @@ void		gtk_spin_button_set_numeric	   (GtkSpinButton  *spin_button,
 
 void		gtk_spin_button_spin		   (GtkSpinButton  *spin_button,
 						    GtkSpinType     direction,
-						    gfloat	    increment);
+						    gdouble	    increment);
 
 void		gtk_spin_button_set_wrap	   (GtkSpinButton  *spin_button,
 						    gboolean	    wrap);
 
-void		gtk_spin_button_set_shadow_type	   (GtkSpinButton  *spin_button,
-						    GtkShadowType   shadow_type);
-
 void		gtk_spin_button_set_snap_to_ticks  (GtkSpinButton  *spin_button,
 						    gboolean	    snap_to_ticks);
 void            gtk_spin_button_update             (GtkSpinButton  *spin_button);
