@@ -83,6 +83,7 @@ struct _GtkStyle
   GdkColor mid[5];
   GdkColor text[5];
   GdkColor base[5];
+  GdkColor text_aa[5];		/* Halfway between text/base */
   
   GdkColor black;
   GdkColor white;
@@ -99,6 +100,7 @@ struct _GtkStyle
   GdkGC *mid_gc[5];
   GdkGC *text_gc[5];
   GdkGC *base_gc[5];
+  GdkGC *text_aa_gc[5];
   GdkGC *black_gc;
   GdkGC *white_gc;
   
@@ -381,6 +383,7 @@ struct _GtkStyleClass
   void (*draw_layout)		(GtkStyle		*style,
 				 GdkWindow		*window,
 				 GtkStateType		 state_type,
+				 gboolean                use_text,
 				 GdkRectangle		*area,
 				 GtkWidget		*widget,
 				 const gchar		*detail,
@@ -588,6 +591,7 @@ void gtk_draw_expander   (GtkStyle        *style,
 void gtk_draw_layout     (GtkStyle        *style,
                           GdkWindow       *window,
                           GtkStateType     state_type,
+			  gboolean         use_text,
                           gint             x,
                           gint             y,
                           PangoLayout     *layout);
@@ -804,6 +808,7 @@ void gtk_paint_expander   (GtkStyle        *style,
 void gtk_paint_layout     (GtkStyle        *style,
                            GdkWindow       *window,
                            GtkStateType     state_type,
+			   gboolean         use_text,
                            GdkRectangle    *area,
                            GtkWidget       *widget,
                            const gchar     *detail,
