@@ -100,11 +100,11 @@ typedef struct _GtkTextLine     GtkTextLine;
 typedef struct _GtkTextLineData GtkTextLineData;
 
 #define GTK_TYPE_TEXT_LAYOUT             (gtk_text_layout_get_type ())
-#define GTK_TEXT_LAYOUT(obj)             (GTK_CHECK_CAST ((obj), GTK_TYPE_TEXT_LAYOUT, GtkTextLayout))
-#define GTK_TEXT_LAYOUT_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_TEXT_LAYOUT, GtkTextLayoutClass))
-#define GTK_IS_TEXT_LAYOUT(obj)          (GTK_CHECK_TYPE ((obj), GTK_TYPE_TEXT_LAYOUT))
-#define GTK_IS_TEXT_LAYOUT_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_TEXT_LAYOUT))
-#define GTK_TEXT_LAYOUT_GET_CLASS(obj)   (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_TEXT_LAYOUT, GtkTextLayoutClass))
+#define GTK_TEXT_LAYOUT(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_TEXT_LAYOUT, GtkTextLayout))
+#define GTK_TEXT_LAYOUT_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_TEXT_LAYOUT, GtkTextLayoutClass))
+#define GTK_IS_TEXT_LAYOUT(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_TEXT_LAYOUT))
+#define GTK_IS_TEXT_LAYOUT_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_TEXT_LAYOUT))
+#define GTK_TEXT_LAYOUT_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_TEXT_LAYOUT, GtkTextLayoutClass))
 
 typedef struct _GtkTextLayout         GtkTextLayout;
 typedef struct _GtkTextLayoutClass    GtkTextLayoutClass;
@@ -114,7 +114,7 @@ typedef struct _GtkTextAttrAppearance GtkTextAttrAppearance;
 
 struct _GtkTextLayout
 {
-  GtkObject parent_instance;
+  GObject parent_instance;
 
   /* width of the display area on-screen,
    * i.e. pixels we should wrap to fit inside. */
@@ -167,7 +167,7 @@ struct _GtkTextLayout
 
 struct _GtkTextLayoutClass
 {
-  GtkObjectClass parent_class;
+  GObjectClass parent_class;
 
   /* Some portion of the layout was invalidated
    */
@@ -239,7 +239,7 @@ struct _GtkTextLineDisplay
 
 extern PangoAttrType gtk_text_attr_appearance_type;
 
-GtkType         gtk_text_layout_get_type    (void) G_GNUC_CONST;
+GType         gtk_text_layout_get_type    (void) G_GNUC_CONST;
 GtkTextLayout*  gtk_text_layout_new         (void);
 void gtk_text_layout_set_buffer             (GtkTextLayout     *layout,
                                              GtkTextBuffer     *buffer);
@@ -339,20 +339,27 @@ gboolean gtk_text_layout_clamp_iter_to_vrange (GtkTextLayout     *layout,
                                                gint               top,
                                                gint               bottom);
 
-void gtk_text_layout_move_iter_to_line_end      (GtkTextLayout *layout,
-                                                 GtkTextIter   *iter,
-                                                 gint           direction);
-void gtk_text_layout_move_iter_to_previous_line (GtkTextLayout *layout,
-                                                 GtkTextIter   *iter);
-void gtk_text_layout_move_iter_to_next_line     (GtkTextLayout *layout,
-                                                 GtkTextIter   *iter);
-void gtk_text_layout_move_iter_to_x             (GtkTextLayout *layout,
-                                                 GtkTextIter   *iter,
-                                                 gint           x);
-void gtk_text_layout_move_iter_visually         (GtkTextLayout *layout,
-                                                 GtkTextIter   *iter,
-                                                 gint           count);
+gboolean gtk_text_layout_move_iter_to_line_end      (GtkTextLayout *layout,
+                                                     GtkTextIter   *iter,
+                                                     gint           direction);
+gboolean gtk_text_layout_move_iter_to_previous_line (GtkTextLayout *layout,
+                                                     GtkTextIter   *iter);
+gboolean gtk_text_layout_move_iter_to_next_line     (GtkTextLayout *layout,
+                                                     GtkTextIter   *iter);
+void     gtk_text_layout_move_iter_to_x             (GtkTextLayout *layout,
+                                                     GtkTextIter   *iter,
+                                                     gint           x);
+gboolean gtk_text_layout_move_iter_visually         (GtkTextLayout *layout,
+                                                     GtkTextIter   *iter,
+                                                     gint           count);
 
+gboolean gtk_text_layout_iter_starts_line           (GtkTextLayout       *layout,
+                                                     const GtkTextIter   *iter);
+
+void     gtk_text_layout_get_iter_at_line           (GtkTextLayout *layout,
+                                                     GtkTextIter    *iter,
+                                                     GtkTextLine    *line,
+                                                     gint            byte_offset);
 
 /* Don't use these. Use gtk_text_view_add_child_at_anchor().
  * These functions are defined in gtktextchild.c, but here
@@ -368,6 +375,9 @@ void gtk_text_child_anchor_unregister_child (GtkTextChildAnchor *anchor,
 void gtk_text_child_anchor_queue_resize     (GtkTextChildAnchor *anchor,
                                              GtkTextLayout      *layout);
 
+void gtk_text_anchored_child_set_layout     (GtkWidget          *child,
+                                             GtkTextLayout      *layout);
+
 void gtk_text_layout_spew (GtkTextLayout *layout);
 
 #ifdef __cplusplus
