@@ -604,9 +604,9 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                    G_OBJECT_CLASS_TYPE (object_class),
                    G_SIGNAL_RUN_LAST,
                    GTK_SIGNAL_OFFSET (GtkTextTagClass, event),
-		   NULL, NULL,
+		   _gtk_boolean_handled_accumulator, NULL,
 		   gtk_marshal_BOOLEAN__OBJECT_BOXED_BOXED,
-                   GTK_TYPE_INT,
+                   G_TYPE_BOOLEAN,
                    3,
                    G_TYPE_OBJECT,
                    GDK_TYPE_EVENT,
