@@ -24,12 +24,14 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include "gtktree.h"
-#include "gtktreeitem.h"
 #include "gtkmain.h"
 #include "gtksignal.h"
 #include "gtklist.h"
 
+#define GTK_ENABLE_BROKEN
+#include "gtktree.h"
+#include "gtktreeitem.h"
+
 enum {
   SELECTION_CHANGED,
   SELECT_CHILD,
@@ -41,12 +43,10 @@ static void gtk_tree_class_init      (GtkTreeClass   *klass);
 static void gtk_tree_init            (GtkTree        *tree);
 static void gtk_tree_destroy         (GtkObject      *object);
 static void gtk_tree_map             (GtkWidget      *widget);
+static void gtk_tree_parent_set      (GtkWidget      *widget,
+				      GtkWidget      *previous_parent);
 static void gtk_tree_unmap           (GtkWidget      *widget);
 static void gtk_tree_realize         (GtkWidget      *widget);
-static void gtk_tree_draw            (GtkWidget      *widget,
-				      GdkRectangle   *area);
-static gint gtk_tree_expose          (GtkWidget      *widget,
-				      GdkEventExpose *event);
 static gint gtk_tree_motion_notify   (GtkWidget      *widget,
 				      GdkEventMotion *event);
 static gint gtk_tree_button_press    (GtkWidget      *widget,
@@ -112,39 +112,13 @@ gtk_tree_class_init (GtkTreeClass *class)
   
   parent_class = gtk_type_class (gtk_container_get_type ());
   
-  tree_signals[SELECTION_CHANGED] =
-    gtk_signal_new ("selection_changed",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkTreeClass, selection_changed),
-		    gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
-  tree_signals[SELECT_CHILD] =
-    gtk_signal_new ("select_child",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkTreeClass, select_child),
-		    gtk_marshal_VOID__POINTER,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
-  tree_signals[UNSELECT_CHILD] =
-    gtk_signal_new ("unselect_child",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkTreeClass, unselect_child),
-		    gtk_marshal_VOID__POINTER,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
-  
-  gtk_object_class_add_signals (object_class, tree_signals, LAST_SIGNAL);
   
   object_class->destroy = gtk_tree_destroy;
   
   widget_class->map = gtk_tree_map;
   widget_class->unmap = gtk_tree_unmap;
+  widget_class->parent_set = gtk_tree_parent_set;
   widget_class->realize = gtk_tree_realize;
-  widget_class->draw = gtk_tree_draw;
-  widget_class->expose_event = gtk_tree_expose;
   widget_class->motion_notify_event = gtk_tree_motion_notify;
   widget_class->button_press_event = gtk_tree_button_press;
   widget_class->button_release_event = gtk_tree_button_release;
@@ -160,6 +134,30 @@ gtk_tree_class_init (GtkTreeClass *class)
   class->selection_changed = NULL;
   class->select_child = gtk_real_tree_select_child;
   class->unselect_child = gtk_real_tree_unselect_child;
+
+  tree_signals[SELECTION_CHANGED] =
+    gtk_signal_new ("selection_changed",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkTreeClass, selection_changed),
+		    gtk_marshal_VOID__VOID,
+		    GTK_TYPE_NONE, 0);
+  tree_signals[SELECT_CHILD] =
+    gtk_signal_new ("select_child",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkTreeClass, select_child),
+		    gtk_marshal_VOID__OBJECT,
+		    GTK_TYPE_NONE, 1,
+		    GTK_TYPE_WIDGET);
+  tree_signals[UNSELECT_CHILD] =
+    gtk_signal_new ("unselect_child",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkTreeClass, unselect_child),
+		    gtk_marshal_VOID__OBJECT,
+		    GTK_TYPE_NONE, 1,
+		    GTK_TYPE_WIDGET);
 }
 
 static GtkType
@@ -172,7 +170,7 @@ static void
 gtk_tree_init (GtkTree *tree)
 {
   tree->children = NULL;
-  tree->root_tree = NULL;
+  tree->root_tree = tree;
   tree->selection = NULL;
   tree->tree_owner = NULL;
   tree->selection_mode = GTK_SELECTION_SINGLE;
@@ -180,7 +178,7 @@ gtk_tree_init (GtkTree *tree)
   tree->current_indent = 0;
   tree->level = 0;
   tree->view_mode = GTK_TREE_VIEW_LINE;
-  tree->view_line = 1;
+  tree->view_line = TRUE;
 }
 
 GtkWidget*
@@ -441,79 +439,6 @@ gtk_tree_destroy (GtkObject *object)
     (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
-static void
-gtk_tree_draw (GtkWidget    *widget,
-	       GdkRectangle *area)
-{
-  GtkTree *tree;
-  GtkWidget *subtree;
-  GtkWidget *child;
-  GdkRectangle child_area;
-  GList *children;
-  
-  
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_TREE (widget));
-  g_return_if_fail (area != NULL);
-  
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      tree = GTK_TREE (widget);
-      
-      children = tree->children;
-      while (children)
-	{
-	  child = children->data;
-	  children = children->next;
-	  
-	  if (gtk_widget_intersect (child, area, &child_area))
-	    gtk_widget_draw (child, &child_area);
-	  
-	  if((subtree = GTK_TREE_ITEM(child)->subtree) &&
-	     GTK_WIDGET_VISIBLE(subtree) &&
-	     gtk_widget_intersect (subtree, area, &child_area))
-	    gtk_widget_draw (subtree, &child_area);
-	}
-    }
-  
-}
-
-static gint
-gtk_tree_expose (GtkWidget      *widget,
-		 GdkEventExpose *event)
-{
-  GtkTree *tree;
-  GtkWidget *child;
-  GdkEventExpose child_event;
-  GList *children;
-  
-  
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_TREE (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-  
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      tree = GTK_TREE (widget);
-      
-      child_event = *event;
-      
-      children = tree->children;
-      while (children)
-	{
-	  child = children->data;
-	  children = children->next;
-	  
-	  if (GTK_WIDGET_NO_WINDOW (child) &&
-	      gtk_widget_intersect (child, &event->area, &child_event.area))
-	    gtk_widget_event (child, (GdkEvent*) &child_event);
-	}
-    }
-  
-  
-  return FALSE;
-}
-
 static void
 gtk_tree_forall (GtkContainer *container,
 		 gboolean      include_internals,
@@ -542,21 +467,44 @@ gtk_tree_forall (GtkContainer *container,
 }
 
 static void
-gtk_tree_map (GtkWidget *widget)
+gtk_tree_unselect_all (GtkTree *tree)
 {
-  GtkTree *tree;
+  GList *tmp_list, *selection;
+  GtkWidget *tmp_item;
+      
+  selection = tree->selection;
+  tree->selection = NULL;
+
+  tmp_list = selection;
+  while (tmp_list)
+    {
+      tmp_item = selection->data;
+
+      if (tmp_item->parent &&
+	  GTK_IS_TREE (tmp_item->parent) &&
+	  GTK_TREE (tmp_item->parent)->root_tree == tree)
+	gtk_tree_item_deselect (GTK_TREE_ITEM (tmp_item));
+
+      gtk_widget_unref (tmp_item);
+
+      tmp_list = tmp_list->next;
+    }
+
+  g_list_free (selection);
+}
+
+static void
+gtk_tree_parent_set (GtkWidget *widget,
+		     GtkWidget *previous_parent)
+{
+  GtkTree *tree = GTK_TREE (widget);
   GtkWidget *child;
   GList *children;
   
-  
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_TREE (widget));
-  
-  GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
-  tree = GTK_TREE (widget);
-  
-  if(GTK_IS_TREE(widget->parent)) 
+  if (widget->parent && GTK_IS_TREE (widget->parent))
     {
+      gtk_tree_unselect_all (tree);
+      
       /* set root tree for this tree */
       tree->root_tree = GTK_TREE(widget->parent)->root_tree;
       
@@ -566,9 +514,34 @@ gtk_tree_map (GtkWidget *widget)
 	tree->indent_value;
       tree->view_mode = GTK_TREE(GTK_WIDGET(tree)->parent)->view_mode;
       tree->view_line = GTK_TREE(GTK_WIDGET(tree)->parent)->view_line;
-    } 
+    }
   else
-    tree->root_tree = tree;
+    {
+      tree->root_tree = tree;
+      
+      tree->level = 0;
+      tree->current_indent = 0;
+    }
+
+  children = tree->children;
+  while (children)
+    {
+      child = children->data;
+      children = children->next;
+      
+      if (GTK_TREE_ITEM (child)->subtree)
+	gtk_tree_parent_set (GTK_TREE_ITEM (child)->subtree, child);
+    }
+}
+
+static void
+gtk_tree_map (GtkWidget *widget)
+{
+  GtkTree *tree = GTK_TREE (widget);
+  GtkWidget *child;
+  GList *children;
+  
+  GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
   
   children = tree->children;
   while (children)
@@ -1058,12 +1031,6 @@ gtk_real_tree_select_child (GtkTree   *tree,
   g_return_if_fail (child != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (child));
 
-  if (!tree->root_tree)
-    {
-      g_warning (G_STRLOC ": unable to select a child in a tree prior to realization");
-      return;
-    }
-  
   root_selection = tree->root_tree->selection;
   
   switch (tree->root_tree->selection_mode)
@@ -1227,7 +1194,7 @@ gtk_tree_set_view_mode (GtkTree       *tree,
 
 void
 gtk_tree_set_view_lines (GtkTree       *tree,
-			 guint          flag) 
+			 gboolean	flag) 
 {
   g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
