@@ -25,12 +25,13 @@
  */
 
 #include "gtklabel.h"
-#include "gtktree.h"
-#include "gtktreeitem.h"
 #include "gtkeventbox.h"
 #include "gtkpixmap.h"
 #include "gtkmain.h"
 #include "gtksignal.h"
+#define GTK_ENABLE_BROKEN
+#include "gtktree.h"
+#include "gtktreeitem.h"
 
 #include "tree_plus.xpm"
 #include "tree_minus.xpm"
@@ -64,19 +65,12 @@ static void gtk_tree_item_size_request  (GtkWidget        *widget,
 					 GtkRequisition   *requisition);
 static void gtk_tree_item_size_allocate (GtkWidget        *widget,
 					 GtkAllocation    *allocation);
-static void gtk_tree_item_draw          (GtkWidget        *widget,
-					 GdkRectangle     *area);
-static void gtk_tree_item_draw_focus    (GtkWidget        *widget);
 static void gtk_tree_item_paint         (GtkWidget        *widget,
 					 GdkRectangle     *area);
 static gint gtk_tree_item_button_press  (GtkWidget        *widget,
 					 GdkEventButton   *event);
 static gint gtk_tree_item_expose        (GtkWidget        *widget,
 					 GdkEventExpose   *event);
-static gint gtk_tree_item_focus_in      (GtkWidget        *widget,
-					 GdkEventFocus    *event);
-static gint gtk_tree_item_focus_out     (GtkWidget        *widget,
-					 GdkEventFocus    *event);
 static void gtk_tree_item_forall        (GtkContainer    *container,
 					 gboolean         include_internals,
 					 GtkCallback      callback,
@@ -90,7 +84,7 @@ static void gtk_real_tree_item_collapse (GtkTreeItem      *item);
 static void gtk_real_tree_item_expand   (GtkTreeItem      *item);
 static void gtk_real_tree_item_collapse (GtkTreeItem      *item);
 static void gtk_tree_item_destroy        (GtkObject *object);
-static void gtk_tree_item_subtree_button_click (GtkWidget *widget);
+static gint gtk_tree_item_subtree_button_click (GtkWidget *widget);
 static void gtk_tree_item_subtree_button_changed_state (GtkWidget *widget);
 
 static void gtk_tree_item_map(GtkWidget*);
@@ -135,13 +129,32 @@ gtk_tree_item_class_init (GtkTreeItemClass *class)
   GtkContainerClass *container_class;
   GtkItemClass *item_class;
 
+  parent_class = gtk_type_class (GTK_TYPE_ITEM);
+  
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   item_class = (GtkItemClass*) class;
   container_class = (GtkContainerClass*) class;
 
-  parent_class = gtk_type_class (gtk_item_get_type ());
-  
+  object_class->destroy = gtk_tree_item_destroy;
+
+  widget_class->realize = gtk_tree_item_realize;
+  widget_class->size_request = gtk_tree_item_size_request;
+  widget_class->size_allocate = gtk_tree_item_size_allocate;
+  widget_class->button_press_event = gtk_tree_item_button_press;
+  widget_class->expose_event = gtk_tree_item_expose;
+  widget_class->map = gtk_tree_item_map;
+  widget_class->unmap = gtk_tree_item_unmap;
+
+  container_class->forall = gtk_tree_item_forall;
+
+  item_class->select = gtk_real_tree_item_select;
+  item_class->deselect = gtk_real_tree_item_deselect;
+  item_class->toggle = gtk_real_tree_item_toggle;
+
+  class->expand = gtk_real_tree_item_expand;
+  class->collapse = gtk_real_tree_item_collapse;
+
   tree_item_signals[EXPAND_TREE] =
     gtk_signal_new ("expand",
 		    GTK_RUN_FIRST,
@@ -156,50 +169,27 @@ gtk_tree_item_class_init (GtkTreeItemClass *class)
 		    GTK_SIGNAL_OFFSET (GtkTreeItemClass, collapse),
 		    gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
-
-  gtk_object_class_add_signals (object_class, tree_item_signals, LAST_SIGNAL);
-
-  object_class->destroy = gtk_tree_item_destroy;
-
-  widget_class->realize = gtk_tree_item_realize;
-  widget_class->size_request = gtk_tree_item_size_request;
-  widget_class->size_allocate = gtk_tree_item_size_allocate;
-  widget_class->draw = gtk_tree_item_draw;
-  widget_class->draw_focus = gtk_tree_item_draw_focus;
-  widget_class->button_press_event = gtk_tree_item_button_press;
-  widget_class->expose_event = gtk_tree_item_expose;
-  widget_class->focus_in_event = gtk_tree_item_focus_in;
-  widget_class->focus_out_event = gtk_tree_item_focus_out;
-  widget_class->map = gtk_tree_item_map;
-  widget_class->unmap = gtk_tree_item_unmap;
-
-  container_class->forall = gtk_tree_item_forall;
-
-  item_class->select = gtk_real_tree_item_select;
-  item_class->deselect = gtk_real_tree_item_deselect;
-  item_class->toggle = gtk_real_tree_item_toggle;
-
-  class->expand = gtk_real_tree_item_expand;
-  class->collapse = gtk_real_tree_item_collapse;
 }
 
 /* callback for event box mouse event */
-static void 
+static gint
 gtk_tree_item_subtree_button_click (GtkWidget *widget)
 {
   GtkTreeItem* item;
   
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_EVENT_BOX (widget));
+  g_return_val_if_fail (widget != NULL, FALSE);
+  g_return_val_if_fail (GTK_IS_EVENT_BOX (widget), FALSE);
   
   item = (GtkTreeItem*) gtk_object_get_user_data (GTK_OBJECT (widget));
   if (!GTK_WIDGET_IS_SENSITIVE (item))
-    return;
+    return FALSE;
   
   if (item->expanded)
     gtk_tree_item_collapse (item);
   else
     gtk_tree_item_expand (item);
+
+  return TRUE;
 }
 
 /* callback for event box state changed */
@@ -576,6 +566,7 @@ gtk_tree_item_draw_lines (GtkWidget *widget)
   GtkTreeItem* item;
   GtkTree* tree;
   guint lx1, ly1, lx2, ly2;
+  GdkGC* gc;
 
   g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (widget));
@@ -586,6 +577,8 @@ gtk_tree_item_draw_lines (GtkWidget *widget)
   if (!tree->view_line)
     return;
 
+  gc = widget->style->text_gc[GTK_STATE_NORMAL];
+
   /* draw vertical line */
   lx1 = item->pixmaps_box->allocation.width;
   lx1 = lx2 = ((lx1 / 2) + (lx1 % 2) + 
@@ -597,7 +590,7 @@ gtk_tree_item_draw_lines (GtkWidget *widget)
     ly2 = (ly2 / 2) + (ly2 % 2);
 
   if (tree != tree->root_tree)
-    gdk_draw_line (widget->window, widget->style->black_gc, lx1, ly1, lx2, ly2);
+    gdk_draw_line (widget->window, gc, lx1, ly1, lx2, ly2);
 
   /* draw vertical line for subtree connecting */
   if(g_list_last(tree->children)->data != (gpointer)widget)
@@ -606,15 +599,14 @@ gtk_tree_item_draw_lines (GtkWidget *widget)
   lx2 += DEFAULT_DELTA;
 
   if (item->subtree && item->expanded)
-    gdk_draw_line (widget->window, widget->style->black_gc,
+    gdk_draw_line (widget->window, gc,
 		   lx2, ly2, lx2, widget->allocation.height);
 
   /* draw horizontal line */
   ly1 = ly2;
   lx2 += 2;
 
-  gdk_draw_line (widget->window, widget->style->black_gc,
-		 lx1, ly1, lx2, ly2);
+  gdk_draw_line (widget->window, gc, lx1, ly1, lx2, ly2);
 
   lx2 -= DEFAULT_DELTA+2;
   ly1 = 0;
@@ -629,7 +621,7 @@ gtk_tree_item_draw_lines (GtkWidget *widget)
 	  lx1 = lx2 -= tree->indent_value;
 	  
 	  if (g_list_last (tree->children)->data != item)
-	    gdk_draw_line (widget->window, widget->style->black_gc, lx1, ly1, lx2, ly2);
+	    gdk_draw_line (widget->window, gc, lx1, ly1, lx2, ly2);
 	  item = GTK_TREE_ITEM (tree->tree_owner);
 	  tree = GTK_TREE (GTK_WIDGET (tree)->parent);
 	} 
@@ -707,39 +699,6 @@ gtk_tree_item_paint (GtkWidget    *widget,
     }
 }
 
-static void
-gtk_tree_item_draw (GtkWidget    *widget,
-		    GdkRectangle *area)
-{
-  GtkBin *bin;
-  GdkRectangle child_area;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_TREE_ITEM (widget));
-  g_return_if_fail (area != NULL);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      bin = GTK_BIN (widget);
-
-      gtk_tree_item_paint (widget, area);
-     
-      if (bin->child && 
-	  gtk_widget_intersect (bin->child, area, &child_area))
-	gtk_widget_draw (bin->child, &child_area);
-
-    }
-}
-
-static void
-gtk_tree_item_draw_focus (GtkWidget *widget)
-{
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_TREE_ITEM (widget));
-
-  gtk_widget_draw(widget, NULL);
-}
-
 static gint
 gtk_tree_item_button_press (GtkWidget      *widget,
 			    GdkEventButton *event)
@@ -754,65 +713,27 @@ gtk_tree_item_button_press (GtkWidget      *widget,
      	&& !GTK_WIDGET_HAS_FOCUS (widget))
       gtk_widget_grab_focus (widget);
 
-  return FALSE;
+  return (event->type == GDK_BUTTON_PRESS && GTK_WIDGET_IS_SENSITIVE(widget));
 }
 
 static gint
 gtk_tree_item_expose (GtkWidget      *widget,
 		      GdkEventExpose *event)
 {
-  GdkEventExpose child_event;
-  GtkBin *bin;
-
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE_ITEM (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
   if (GTK_WIDGET_DRAWABLE (widget))
     {
-      bin = GTK_BIN (widget);
-      
       gtk_tree_item_paint (widget, &event->area);
 
-      child_event = *event;
-      if (bin->child && GTK_WIDGET_NO_WINDOW (bin->child) &&
-	  gtk_widget_intersect (bin->child, &event->area, &child_event.area))
-	gtk_widget_event (bin->child, (GdkEvent*) &child_event);
+      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
    }
 
   return FALSE;
 }
 
-static gint
-gtk_tree_item_focus_in (GtkWidget     *widget,
-			GdkEventFocus *event)
-{
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_TREE_ITEM (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-
-  return FALSE;
-}
-
-static gint
-gtk_tree_item_focus_out (GtkWidget     *widget,
-			 GdkEventFocus *event)
-{
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_TREE_ITEM (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-
-  return FALSE;
-}
-
 static void
 gtk_real_tree_item_select (GtkItem *item)
 {    
