@@ -70,6 +70,7 @@ struct _GtkWindow
 
   GtkWindowType type : 4;
   guint has_user_ref_count : 1;
+  guint has_focus : 1;
   guint allow_shrink : 1;
   guint allow_grow : 1;
   guint auto_shrink : 1;
@@ -113,6 +114,13 @@ struct _GtkWindowClass
 			    GtkWidget *focus);
   gboolean (* frame_event) (GtkWidget *widget,
 			    GdkEvent  *event);
+
+  /* G_SIGNAL_ACTION signals for keybindings */
+
+  void     (* activate_focus)          (GtkWindow       *window);
+  void     (* activate_default)        (GtkWindow       *window);
+  void     (* move_focus)              (GtkWindow       *window,
+                                        GtkDirectionType direction);  
 };
 
 
@@ -140,10 +148,6 @@ void       gtk_window_set_type_hint            (GtkWindow           *window,
 						GdkWindowTypeHint    hint);
 void       gtk_window_set_destroy_with_parent  (GtkWindow           *window,
                                                 gboolean             setting);
-void       gtk_window_set_decorations_hint     (GtkWindow	    *window,
-                                                GdkWMDecoration      decorations);
-void       gtk_window_set_functions_hint       (GtkWindow	    *window,
-                                                GdkWMFunction	     functions);
 
 void       gtk_window_set_resizeable           (GtkWindow           *window,
                                                 gboolean             resizeable);
@@ -212,6 +216,7 @@ void       gtk_window_set_policy               (GtkWindow           *window,
 						gint                 allow_shrink,
 						gint                 allow_grow,
 						gint                 auto_shrink);
+#endif
 /* The following differs from gtk_widget_set_usize, in that
  * gtk_widget_set_usize() overrides the requisition, so sets a minimum
  * size, while this only sets the size requested from the WM.
@@ -219,7 +224,6 @@ void       gtk_window_set_policy               (GtkWindow           *window,
 void       gtk_window_set_default_size         (GtkWindow           *window,
 						gint                 width,
 						gint                 height);
-#endif
 
 /* --- internal functions --- */
 void       gtk_window_set_focus                (GtkWindow           *window,
